const path = require('path');

/**
 * Configuring the webpack configure for the test cases
 *
 * */
const webpackConfig = require('./webpack.config');
webpackConfig.entry = null;
webpackConfig.devtool = 'inline-source-map';
webpackConfig.mode = 'development';
webpackConfig.module.rules.push({
	test: /\.js$|\.es6$/,
	use: {
		loader: 'istanbul-instrumenter-loader',
		options: {esModules: true}
	},
	enforce: 'post',
	exclude:[
		/node_modules|\.spec\.js$/,
		path.resolve(__dirname, 'src/libs/'),
		path.resolve(__dirname, 'src/editor-app/libs')
	]
});

/**
 * Adding es6 support to the spec files
 * */
webpackConfig.module.rules.push({
	test: /\.js$/,
	use: {
		loader: 'babel-loader'
	},
	include: [
		path.resolve(__dirname, 'tests/*')
	]
})


/**
 *End of the webpack configuration
 * */
module.exports = (config) => {
	config.set({
		frameworks: ['jasmine'],

		files: [
			'tests/index.spec.js',
		],

		reporters: ['spec', 'coverage-istanbul'], 

		preprocessors: {
			'tests/index.spec.js': ['webpack'],
			'src/**/*.js': ['webpack', 'sourcemap'],
			'src/root.es6': ['webpack', 'sourcemap']
		},


		coverageIstanbulReporter: {
			reports: ['html', 'teamcity', 'text-summary'],
			dir: path.resolve(__dirname, 'coverage'),
			'report-config': {
				html: {
					subdir: 'view'
				},
				teamcity: {
					file: 'teamcity'
				}
			}
		},

		webpack: webpackConfig,

		webpackMiddleware: {
			stats: 'errors-only'
		},

		browsers: [
			'Chrome',
		],
		
	});
}

