/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * Assignment
 */
'use strict';

angular.module('workflowModeler').controller('WorkflowAssignmentCtrl', ['$scope', '$modal', function ($scope, $modal) {

    // Config for the modal window
    var opts = {
        template: 'editor-app/configuration/properties/assignment-popup.html',
        scope: $scope
    };

    // Open the dialog
    _internalCreateModal(opts, $modal, $scope);
}]);

angular.module('workflowModeler').directive('workflowGridCellEdit', ['$timeout', function ($timeout) {
    return {
        restrict: 'E',
        require: ['^ngModel', '?^uiGrid'],
        scope: {
            ngModel: '='
        },
        template: '<div class="workflow-grid-cell-edit"><input type="text" class="form-control" ng-model="value" ng-pattern="/^[a-zA-Z0-9_]*$/">' +
            '<div class="ot-icons ot-icon-edit_cancel" role="button" tabindex="0"  ng-click="cancelEdit($event)"></div> </div>',
        link: function (scope, element, attrs, controls) {
            var control = controls[0];
            var gridCntrl = controls[1];
            scope.value = scope.ngModel;
            var input = element.find('input');
            if (input) {
                input.focus();
                scope.inputForm = element.find('input').controller('ngModel');
            }
            scope.$watch('value', function (newValue, oldValue) {
                if (newValue !== oldValue) {
                    control.$modelValue = newValue;
                    control.$setViewValue(newValue);
                    scope.ngModel = newValue;
                }
            });


            element.on('keydown', function ($event) {
                if ($event.keyCode === 13 || $event.keyCode === 9) {
                    scope.stopEdit($event);
                }
            });

            scope.cancelEdit = function () {
                $timeout(function () {
                    scope.$emit('uiGridEventCancelCellEdit');
                });
            }

            scope.stopEdit = function (evt) {
                if (scope.inputForm && !scope.inputForm.$valid) {
                    evt.stopPropagation();
                    scope.$emit('uiGridEventCancelCellEdit');
                }
                else {
                    scope.$emit('uiGridEventEndCellEdit');
                }
            };
        }
    }
}]);

angular.module('workflowModeler').directive("percentageInput", ['$templateCache', '$timeout', function ($templateCache, $timeout) {
    return {
        restrict: 'E',
        require: 'ngModel',
        scope: {
            ngModel: '=',
        },
        template: $templateCache.get('editor-app/configuration/properties/percentage-input-template.html'),
        link: function (scope, element, attrs, control) {
            var inputCntrl = element.find('input').controller('ngModel');
            scope.value = scope.ngModel;

            scope.$watch('value', function (newValue, oldValue) {
                if (newValue !== oldValue) {
                    scope.ngModel = newValue;
                    control.$setViewValue(newValue);
                }
            });

            function parseViewValue(value) {
                if (value) {
                    var transformedInput = value.replace(/[^0-9]/g, '');
                    if (transformedInput) {
                        transformedInput = parseInt(transformedInput);
                        if (transformedInput >= 0 && transformedInput <= 100) {
                            var viewValue = transformedInput + '%';
                            $timeout(function() {
                                inputCntrl.$viewValue = viewValue;
                                inputCntrl.$render();
                            });
                            return transformedInput;
                        } else {
                            $timeout(function () {
                                inputCntrl.$viewValue = inputCntrl.$$rawModelValue + '%';
                                inputCntrl.$render();
                            })
                            return inputCntrl.$$rawModelValue;
                        }
                    } else {
                        $timeout(function() {
                            inputCntrl.$viewValue = '';
                            inputCntrl.$render();
                        });
                        return '';
                    }
                }
            }

            function formatModelValue(value) {
                if (!value) {
                    inputCntrl.$modelValue = '';
                    return '';
                }
                value = '' + value;
                if (value.charAt(value.length - 1) === '%') {
                    value = value.substring(0, str.length - 1);
                }
                if (value) {
                    var modelValue = parseInt(value);
                    if (isFinite(modelValue) && modelValue >= 0 && modelValue <= 100) {
                        inputCntrl.$modelValue = modelValue;
                        return modelValue + '%';
                    }
                }
                return undefined;
            }
            inputCntrl.$parsers.push(parseViewValue);
            inputCntrl.$formatters.push(formatModelValue);

        },
        controller: ['$scope', function percentageInputCntrl($scope) {


            $scope.increment = function ($event) {
                $event.stopPropagation();
                if (isFinite($scope.value) && $scope.value < 100) {
                    $scope.value++;
                } else if ($scope.value === 100) {
                    $scope.value = 100;
                }
            }

            $scope.decrement = function ($event) {
                $event.stopPropagation();
                if (isFinite($scope.value) && $scope.value > 0) {
                    $scope.value--;
                } else {
                    $scope.value = 0;
                }
            }
        }]
    };
}]);

angular.module('workflowModeler').controller('WorkflowAssignmentPopupCtrl',
    ['$rootScope', '$q', '$timeout', '$scope', '$translate', '$http', 'editorManager', 'toastService', function ($rootScope, $q, $timeout, $scope, $translate, $http, editorManager, toastService) {

        const MULTIINSTANCE_LOOP_VAR_NAME = "assignment";
        const MULTIINSTANCE_LOOP_ASSIGNEE_VAR = "assignee";
        const MULTIINSTANCE_LOOP_CANDIDATE_USERS_VAR = "candidateUsers";
        const MULTIINSTANCE_LOOP_CANDIDATE_GROUPS_VAR = "candidateGroups";

        $scope.tabs = [
            {
                id: 'assignments',
                title: 'Assignments'
            },
            {
                id: 'outcomes',
                title: 'Outcomes',
                disable: true
            },
            // {
            //     id: 'instructions',
            //     title: 'Instrcutions'
            // }
        ];

        function enableOutcomesTab() {
            if ($scope.participants.length > 0 || $scope.assignToInitiator || $scope.enableMultiInstanceVariable) {
                $scope.tabs[1].disable = false;
            } else {
                $scope.tabs[1].disable = true;
            }
        }

        function updateMultiInstanceProps() {
            $scope.isMultiAssignmentViaTable = $scope.participants.length > 1;
            $scope.isMultiInstance = $scope.isMultiAssignmentViaTable || $scope.enableMultiInstanceVariable;
        }

        $scope.$watch('participants.length', enableOutcomesTab);

        $scope.$watch('assignToInitiator', enableOutcomesTab);

        $scope.$watch('enableMultiInstanceVariable', enableOutcomesTab);

        $scope.$watch('enableMultiInstanceVariable', updateMultiInstanceProps);

        $scope.taskTypeProperty = getPropertyObject('oryx-tasktype');
        $scope.taskOutcomeVarProperty = getPropertyObject('oryx-outcomevariable');
        $scope.taskOutcomesOptionsProperty = getPropertyObject('oryx-possibleoutcomes');
        $scope.completionConditionProperty = getPropertyObject('oryx-multiinstance_condition');

        $scope.model = {};
        $scope.taskType = 'default';
        if ($scope.taskTypeProperty.value) {
            $scope.taskType = $scope.taskTypeProperty.value;
        }

        $scope.model.outcomeVariable = '';
        if ($scope.taskOutcomeVarProperty.value) {
            $scope.model.outcomeVariable = $scope.taskOutcomeVarProperty.value;
        }

    
        $scope.deliveryTypes = [
            { id: "Single Instance", title: $translate.instant('PROPERTY.ASSIGNMENT.DROPDOWN.SINGLEINSTANCE') },
            { id: "Multi Instance", title: $translate.instant('PROPERTY.ASSIGNMENT.DROPDOWN.MULTIINSTANCE') }
        ];
        $scope.sequenceTypes = [
            { id: "Sequential", title: $translate.instant('PROPERTY.ASSIGNMENT.DROPDOWN.SERIAL') },
            { id: "Parallel", title: $translate.instant('PROPERTY.ASSIGNMENT.DROPDOWN.PARALLEL') }
        ];

        //$scope.deliveryType = $scope.deliveryTypes[0].id;
        $scope.isParallelTask = false;
        $scope.isMultiInstance = false;
        $scope.isMultiAssignmentViaTable = false;
        $scope.enableMultiInstanceVariable = false;
        $scope.initiatorCanCompleteTask = false;
        $scope.assignToInitiator = false;
        $scope.participants = [];
        var loadDataPropertiesDataToAssignments = function () {
            if ($scope.property.value !== undefined && $scope.property.value !== null
                && $scope.property.value.assignment !== undefined
                && $scope.property.value.assignment !== null) {

                if ($scope.property.value.assignment.constructor == String) {
                    $scope.assignment = JSON.parse($scope.property.value.assignment);
                }
                else {
                    // Note that we clone the json object rather then setting it directly,
                    // this to cope with the fact that the user can click the cancel button and no changes should have happened
                    $scope.assignment = angular.copy($scope.property.value.assignment);
                }

                // Multi instance assignment data loading.
                // We have to find the data in Mulit instance colletion property.
                // We have to convert data as per UI (inputs, grid) to show.
                convertDataPropertiesDataToParticipantsGrid();
                updateMultiInstanceProps();
                var sequence = $scope.selectedShape.properties.get("oryx-multiinstance_type");
                if (sequence != "" && sequence == $scope.sequenceTypes[1].id) {
                    $scope.isParallelTask = true;
                } else {
                    $scope.isParallelTask = false;
                }

                for (var i = 0; i < $scope.participants.length; i++) {
                    var assignment = $scope.participants[i];
                    if (assignment.value !== undefined && assignment.value !== '') {
                        assignment.value = assignment.value;
                    }
                }
            } else {
                $scope.participants = [];
            }
        }

        var convertDataPropertiesDataToParticipantsGrid = function () {
            var dataProperties = $scope.selectedShape.properties.get("oryx-multiinstance_collection");
            if (dataProperties != "") { // Multi instance.
                if (typeof dataProperties == 'string') {
                    if(dataProperties.startsWith("${") && dataProperties.endsWith("}")) {
                        $scope.enableMultiInstanceVariable = true;
                        $scope.multiInstanceCollectionVariable = dataProperties.substr(2,dataProperties.length - 3);
                    } else {
                        dataProperties = JSON.parse(dataProperties);
                        for (var i = 0; i < dataProperties.length; i++) {
                            $scope.participants.push({
                                assignee: dataProperties[i].assignee,
                                candidateUsers: dataProperties[i].candidateUsers,
                                candidateGroups: dataProperties[i].candidateGroups,
                            });
                        }
                    }
                }
            } else { // Single Assignment data
                $scope.initiatorCanCompleteTask = $scope.property.value.assignment.initiatorCanCompleteTask || false;
                $scope.assignToInitiator = $scope.property.value.assignment.type == 'idm' ? true : false;
                $scope.participants = [];
                if (!$scope.assignToInitiator) {
                    var assignee = $scope.assignment.assignee ? $scope.assignment.assignee : '';
                    var users = $scope.assignment.candidateUsers && $scope.assignment.candidateUsers.length ? concatAllValues($scope.assignment.candidateUsers) : '';
                    var groups = $scope.assignment.candidateGroups && $scope.assignment.candidateGroups.length ? concatAllValues($scope.assignment.candidateGroups) : '';
                    $scope.participants.push({
                        assignee: assignee,
                        candidateUsers: users,
                        candidateGroups: groups
                    });
                }
            }
        }
        loadDataPropertiesDataToAssignments();
        
        function concatAllValues(userOrGroupArray){
            var userOrGroupValues = "";
            for(var i = 0; i < userOrGroupArray.length; i++) {
                userOrGroupValues += userOrGroupArray[i].value + ",";
            }
            return userOrGroupValues.substr(0, userOrGroupValues.length - 1);
        }

        $scope.selectedAssignment = undefined;
        $scope.translationsRetrieved = false;
        $scope.labels = {};

        var assigneePromise = $translate('PROPERTY.ASSIGNMENT.ASSIGNEE');
        var candiateUsersPromise = $translate('PROPERTY.ASSIGNMENT.CANDIDATE_USERS');
        var candidateGroupsPromise = $translate('PROPERTY.ASSIGNMENT.CANDIDATE_GROUPS');

        $q.all([assigneePromise, candiateUsersPromise, candidateGroupsPromise]).then(function (results) {
            $scope.labels.assigneePromise = results[0];
            $scope.labels.candiateUsersPromise = results[1];
            $scope.labels.candidateGroupsPromise = results[2];
            $scope.translationsRetrieved = true;

            // Config for grid
            $scope.gridOptions = {
                data: $scope.participants,
                headerRowHeight: 28,
                rowHeight: 40,
                enableRowSelection: true,
                enableRowHeaderSelection: false,
                multiSelect: false,
                modifierKeysToMultiSelect: false,
                enableHorizontalScrollbar: 0,
                enableColumnMenus: false,
                enableSorting: false,
                enableCellEdit: false,
                columnDefs: [{ field: 'rowIndex', displayName: '#', headerCellClass:'gird-header-align-center', maxWidth: 40, cellTemplate: '<span style="display: inline-block;width: 100%;height: 100%;padding-left: 16px;padding-top: 10px;font-size: 14px;color: #000000;">{{rowRenderIndex  + 1}}</span>' },
                    { field: 'assignee', displayName: $scope.labels.assigneeLabel, cellTemplate: '<input type="text" ng-model="row.entity.assignee" class="form-control grid-custom-row" aria-label="assignee" ng-disabled="grid.appScope.assignToInitiator || grid.appScope.enableMultiInstanceVariable" ng-style="(grid.appScope.assignToInitiator || grid.appScope.enableMultiInstanceVariable) && {opacity:0.4}"/>' },
                    { field: 'candidateUsers', displayName: $scope.labels.candidateUsersLabel, cellTemplate: '<input type="text" ng-model="row.entity.candidateUsers" class="form-control grid-custom-row" aria-label="candidate user" ng-disabled="grid.appScope.assignToInitiator || grid.appScope.enableMultiInstanceVariable" ng-style="(grid.appScope.assignToInitiator || grid.appScope.enableMultiInstanceVariable) && {opacity:0.4}"/>' },
                    { field: 'candidateGroups', displayName: $scope.labels.candidateGroupsLabel, cellTemplate: '<input type="text" ng-model="row.entity.candidateGroups" class="form-control grid-custom-row" aria-label="candidate group" ng-disabled="grid.appScope.assignToInitiator || grid.appScope.enableMultiInstanceVariable" ng-style="(grid.appScope.assignToInitiator || grid.appScope.enableMultiInstanceVariable) && {opacity:0.4}"/>' }]
            };

            $scope.gridOptions.onRegisterApi = function (gridApi) {
                //set gridApi on scope
                $scope.gridApi = gridApi;
                gridApi.selection.on.rowSelectionChanged($scope, function (row) {
                    $scope.selectedAssignment = row.entity;
                });
            };
        });

        $scope.participantsDetailsChanged = function () {
            if ($scope.selectedAssignment && $scope.selectedAssignment.value !== '') {
                $scope.selectedAssignment.assignee = $scope.selectedAssignment.assignee;
                $scope.selectedAssignment.candidateUsers = $scope.selectedAssignment.candidateUsers;
                $scope.selectedAssignment.candidateGroups = $scope.selectedAssignment.candidateGroups;
            } else if ($scope.selectedAssignment) {
                $scope.selectedAssignment.assignee = '';
                $scope.selectedAssignment.candidateUsers = '';
                $scope.selectedAssignment.candidateGroups = '';
            }
        };

        // Click handler for add button
        $scope.addNewAssignment = function () {
            var newAssignment = {
                assignee: '',
                candidateUsers: '',
                candidateGroups: '',
            };
            $scope.participants.push(newAssignment);
            updateMultiInstanceProps();
            $timeout(function () {
                $scope.gridApi.selection.toggleRowSelection(newAssignment);
            });
        };

        // Key press handler for Assignment add button
        $scope.addAssignKeyPress = function (event) {
            if (event.keyCode == 13 || event.keyCode == 32) {
                $scope.addNewAssignment();
            }
        };
        // Click handler for remove button
        $scope.removeAssignment = function () {
            var selectedItems = $scope.gridApi.selection.getSelectedRows();
            if (selectedItems && selectedItems.length > 0) {
                var index = $scope.participants.indexOf(selectedItems[0]);
                $scope.gridApi.selection.toggleRowSelection(selectedItems[0]);

                $scope.participants.splice(index, 1);

                if ($scope.participants.length == 0) {
                    $scope.selectedAssignment = undefined;
                }
                updateMultiInstanceProps();
                $timeout(function () {
                    if ($scope.participants.length > 0) {
                        $scope.gridApi.selection.toggleRowSelection($scope.participants[0]);
                    }
                });
            }
        };

        // Key press handler for Assignment remove button
        $scope.removeAssignKeyPress = function (event) {
            if (event.keyCode == 13 || event.keyCode == 32) {
                $scope.removeAssignment();
            }
        };
        // Click handler for up button
        $scope.moveAssignmentUp = function () {
            var selectedItems = $scope.gridApi.selection.getSelectedRows();
            if (selectedItems && selectedItems.length > 0) {
                var index = $scope.participants.indexOf(selectedItems[0]);
                if (index != 0) { // If it's the first, no moving up of course
                    var temp = $scope.participants[index];
                    $scope.participants.splice(index, 1);
                    $timeout(function () {
                        $scope.participants.splice(index + -1, 0, temp);
                        $timeout(function () {
                            $scope.gridApi.selection.toggleRowSelection(temp);
                        });
                    });
                }
            }
        };

        // Key press handler for Move up Assignment button
        $scope.moveUpKeyPress = function (event) {
            if (event.keyCode == 13 || event.keyCode == 32) {
                $scope.moveAssignmentUp();
            }
        };
        // Click handler for Move down Assignment button
        $scope.moveAssignmentDown = function () {
            var selectedItems = $scope.gridApi.selection.getSelectedRows();
            if (selectedItems && selectedItems.length > 0) {
                var index = $scope.participants.indexOf(selectedItems[0]);
                if (index != $scope.participants.length - 1) { // If it's the last element, no moving down of course
                    var temp = $scope.participants[index];
                    $scope.participants.splice(index, 1);
                    $timeout(function () {
                        $scope.participants.splice(index + 1, 0, temp);
                        $timeout(function () {
                            $scope.gridApi.selection.toggleRowSelection(temp);
                        });
                    });
                }
            }
        };

        // Key press handlerfor  Move down Assignment button
        $scope.moveDownKeyPress = function (event) {
            if (event.keyCode == 13 || event.keyCode == 32) {
                $scope.moveAssignmentDown();
            }
        };

        var prepareAndSetMultiAssignmentCollection = function() {
            var dataproperty_values = [];
            for (var i = 0; i < $scope.participants.length; i++) {
                dataproperty_values.push({ "assignee": $scope.participants[i].assignee, 
                    "candidateUsers": $scope.participants[i].candidateUsers, 
                    "candidateGroups": $scope.participants[i].candidateGroups });
            }
            $scope.selectedShape.properties.set("oryx-multiinstance_collection", JSON.stringify(dataproperty_values));
        }

        var setAssignmentForMultiInstanceTask = function() {
            $scope.property.value.assignment.assignee = "${" + MULTIINSTANCE_LOOP_VAR_NAME + "." + MULTIINSTANCE_LOOP_ASSIGNEE_VAR + "}";
            $scope.property.value.assignment.candidateUsers = [
                {
                    "value": "${" + MULTIINSTANCE_LOOP_VAR_NAME + "." + MULTIINSTANCE_LOOP_CANDIDATE_USERS_VAR + "}"
                }
            ],
            $scope.property.value.assignment.candidateGroups = [
                {
                    "value": "${" + MULTIINSTANCE_LOOP_VAR_NAME + "." + MULTIINSTANCE_LOOP_CANDIDATE_GROUPS_VAR + "}"
                }
            ]
        }
        // Update assignment data to "multiinstance_collection".
        // Convert data as per requirement and push.
        var updateDataPropertiesWithAssigmentMultiInstance = function () {
            $scope.selectedShape.properties.set("oryx-multiinstance_variable", MULTIINSTANCE_LOOP_VAR_NAME);
            $scope.selectedShape.properties.set("multiinstance_collection_class", $scope.enableMultiInstanceVariable ? "" : "$JSONHANDLER");
            var sequance = $scope.isParallelTask ? $scope.sequenceTypes[1].id : $scope.sequenceTypes[0].id;
            $scope.updatePropertyInModel({ "key": "oryx-multiinstance_type", "title": "BPMN.PROPERTYPACKAGES.MULTIINSTANCE_TYPEPACKAGE.MULTIINSTANCE_TYPE.TITLE", "description": "BPMN.PROPERTYPACKAGES.MULTIINSTANCE_TYPEPACKAGE.MULTIINSTANCE_TYPE.DESCRIPTION", "type": "workflow-multiinstance", "mode": "write", "hidden": true, "value": sequance, "readonly": false, "readModeTemplateUrl": "editor-app/configuration/properties/default-value-display-template.html", "writeModeTemplateUrl": "editor-app/configuration/properties/multiinstance-property-write-template.html", "hasReadWriteMode": true });

            if($scope.enableMultiInstanceVariable) {
                $scope.selectedShape.properties.set("oryx-multiinstance_collection", "${" + $scope.multiInstanceCollectionVariable + "}");
            } else {
                prepareAndSetMultiAssignmentCollection();
            }
            setAssignmentForMultiInstanceTask();
        }

        var deleteAssignmentFromDataProperties = function () {
            $scope.selectedShape.properties.set("multiinstance_collection_class", "");
            $scope.selectedShape.properties.set("oryx-multiinstance_collection", "");
            $scope.selectedShape.properties.set("oryx-multiinstance_variable", "");
            $scope.updatePropertyInModel({ "key": "oryx-multiinstance_type", "title": "BPMN.PROPERTYPACKAGES.MULTIINSTANCE_TYPEPACKAGE.MULTIINSTANCE_TYPE.TITLE", "description": "BPMN.PROPERTYPACKAGES.MULTIINSTANCE_TYPEPACKAGE.MULTIINSTANCE_TYPE.DESCRIPTION", "type": "workflow-multiinstance", "mode": "write", "hidden": true, "value": "", "readonly": false, "readModeTemplateUrl": "editor-app/configuration/properties/default-value-display-template.html", "writeModeTemplateUrl": "editor-app/configuration/properties/multiinstance-property-write-template.html", "hasReadWriteMode": true });
        }

        // Click handler for save button
        $scope.save = function () {
            if ($scope.clearValidationAlert) {
                $scope.clearValidationAlert();
            }
            if (!validateInputs()) {
                return ;
            }
            $scope.property.value = {};
            $scope.property.value.assignment = {};
            deleteAssignmentFromDataProperties();
            if ($scope.participants.length > 1 || $scope.enableMultiInstanceVariable) {
                updateDataPropertiesWithAssigmentMultiInstance();
            } else if ($scope.participants.length == 1 || $scope.assignToInitiator) {
                if ($scope.assignToInitiator != true) {
                    $scope.property.value.assignment.assignee = $scope.participants[0].assignee || '';
                    $scope.property.value.assignment.candidateUsers = generateAllValues($scope.participants[0].candidateUsers); //[{ "value": $scope.participants[0].candidateUsers }];
                    $scope.property.value.assignment.candidateGroups = generateAllValues($scope.participants[0].candidateGroups); //[{ "value": $scope.participants[0].candidateGroups }];
                }
                $scope.property.value.assignment.type = $scope.assignToInitiator ? 'idm' : 'static';
                if ($scope.property.value.assignment.type == 'idm') {
                    $scope.property.value.assignment.idm = {}
                    $scope.property.value.assignment.idm.type = "initiator";
                }
                $scope.property.value.assignment.initiatorCanCompleteTask = $scope.initiatorCanCompleteTask || false;
            } else { // when update grid with zero rows or remove all assignments.
                deleteAssignmentFromDataProperties();
                $scope.property.value.assignment = undefined;
            }
            $scope.updatePropertyInModel($scope.property);
        
            $scope.taskTypeProperty.value = $scope.taskType;
            $scope.updatePropertyInModel($scope.taskTypeProperty);

            if ($scope.participants.length > 0 || $scope.assignToInitiator || $scope.enableMultiInstanceVariable) {
                updateOutcomeVariable();
                updateOutcomeOptions();
                updateCompletionCondition();
            } else {
                resetOutcomeVariable();
                resetOutcomeOptions();
                resetCompletionCondition();
            }
            $scope.close();
        }



        // Single assignment and it might have multiple candidate users or groups by comma separated.
        function generateAllValues(usersOrGroups) {
            var usersOrGroupsArray = [];
            usersOrGroups = usersOrGroups.split(",");
            for(var i = 0; i < usersOrGroups.length; i++){
                usersOrGroupsArray.push({ "value": usersOrGroups[i].trim()})
            }
            return usersOrGroupsArray;
        }
        
        function validateInputs() {
            return validateDynamicTaskAssignmentVar() && validateOutcomes();
        }

        function validateDynamicTaskAssignmentVar() {
            if($scope.enableMultiInstanceVariable && !$scope.multiInstanceCollectionVariable) {
                    toastService.error($translate.instant('PROPERTY.ASSIGNMENT.MULTIINSTANCE.VARIABLE_NAME_NULL'));
                    return false;
            }
            return true;
        }

        function validateOutcomes() {
            if ($scope.taskType === 'default') {
                var outcomes = [];
                var arr = $scope.outcomesArray;
                for (var i = 0; i < arr.length; i++) {
                    var value = $scope.outcomesArray[i].outcome;
                    if (outcomes.indexOf(value) !== -1) {
                        $scope.clearValidationAlert = toastService.error($translate.instant('TASKOUTCOME.VALIDATION.UNIQUEOUTCOMES'));
                        return false;
                    }
                    outcomes.push(value);
                }
            }

            if ($scope.taskType === 'approval') {
                var outcomes = [];
                var arr = $scope.outcomeApprovalArray;
                for (var i = 0; i < arr.length; i++) {
                    var value = $scope.outcomeApprovalArray[i].customizedValue;
                    if (outcomes.indexOf(value) !== -1) {
                        $scope.clearValidationAlert = toastService.error($translate.instant('TASKOUTCOME.VALIDATION.APPROVALTASKUNIQUEOUTCOMES', {value: 'Customized values of outcomes'}));
                        return false;
                    }
                    outcomes.push(value);
                }
            }
            
            return true;
        }

        function resetOutcomeVariable() {
            $scope.taskOutcomeVarProperty.value = '';
            $scope.updatePropertyInModel($scope.taskOutcomeVarProperty);
        }

        function resetOutcomeOptions() {
            $scope.taskOutcomesOptionsProperty.value = '';
            $scope.updatePropertyInModel($scope.taskOutcomesOptionsProperty);
        }

        function resetCompletionCondition() {
            $scope.completionConditionProperty.value = '';
            $scope.updatePropertyInModel($scope.completionConditionProperty);
        }

        function updateOutcomeVariable() {
            $scope.taskOutcomeVarProperty.value = $scope.model.outcomeVariable;
            $scope.updatePropertyInModel($scope.taskOutcomeVarProperty);
        }

        function updateOutcomeOptions() {

            if ($scope.taskType === 'approval') {
                var outcome = prepareApprovalOutcome();
                $scope.taskOutcomesOptionsProperty.value = outcome;
                $scope.updatePropertyInModel($scope.taskOutcomesOptionsProperty);
                return;
            }
            if ($scope.taskType === 'default' && $scope.outcomesArray && $scope.outcomesArray.length > 0) {
                var outcome = $scope.outcomesArray.map(function (obj) {
                    return obj.outcome;
                }).join(',');
                $scope.taskOutcomesOptionsProperty.value = outcome;
                $scope.updatePropertyInModel($scope.taskOutcomesOptionsProperty);
            }
        }

        function prepareApprovalOutcome() {
            var approve = $scope.outcomeApprovalArray[0];
            var reject = $scope.outcomeApprovalArray[1];
            return approve.outcome + ":" + approve.customizedValue + "," + reject.outcome + ":" + reject.customizedValue;
        }

        function updateCompletionCondition() {
            var outcomeOption = '';
            var completionCondition = '';
            if ($scope.isMultiInstance) {
                var arr = $scope.outcomesArray;
                if ($scope.taskType === 'approval') {
                    arr = $scope.outcomeApprovalArray;
                }
                for (var iOutcome = 0; iOutcome < arr.length; iOutcome++) {
                    var outcome = arr[iOutcome];
                    if (outcome.conditionForCompletion) {
                        outcomeOption = outcome.outcome;
                        if ($scope.taskType === 'approval') {
                            outcomeOption = outcome.customizedValue;
                        }
                        completionCondition = outcome.conditionForCompletion;
                        break;
                    }
                }
            }
            if (completionCondition) {
                var value = getCompletionCondition($scope.model.outcomeVariable, outcomeOption, completionCondition);
                $scope.completionConditionProperty.value = value;
                $scope.updatePropertyInModel($scope.completionConditionProperty);
            } else {
                $scope.completionConditionProperty.value = '';
                $scope.updatePropertyInModel($scope.completionConditionProperty);
            }
        }

        function getCompletionCondition(varName, option, completionCondition) {
            if (!varName) {
                varName = '$OUTCOME';
            }
            return "${(" + varName + "." + option + "/nrOfInstances)*100 >=" + completionCondition + "}";
        }

        $scope.cancel = function () {
            $scope.close();
        };

        // Close button handler
        $scope.close = function () {
            if ($scope.clearValidationAlert) {
                $scope.clearValidationAlert();
            }
            $scope.property.mode = 'read';
            $scope.$hide();
        };

        // Key press handler for close button
        $scope.closeOnKeyPress = function (event) {
            if (event.keyCode == 13 || event.keyCode == 32) {
                $scope.close();
            }
        };

        // to find the taskType Property
        function getPropertyObject(propertyName) {
            if ($scope.selectedItem) {
                var properties = $scope.selectedItem.properties;
                for (var iProp = 0; iProp < properties.length; iProp++) {
                    if (propertyName === properties[iProp].key) {
                        return properties[iProp];
                    }
                }
            }
        }

        //Code for the outcomes

        function populateOutcomeOptions() {
            var varRegEx = /\.([a-zA-Z0-9_]+)\//;
            var percentageRegEx = /=([0-9]+)}$/;
            var outcomes = $scope.taskOutcomesOptionsProperty.value.split(',');
            var completionCondition = $scope.completionConditionProperty.value;
            var conditionON = getConditionValues(completionCondition, varRegEx);
            var conditionValue = getConditionValues(completionCondition, percentageRegEx);
            if ($scope.taskType === 'default') {
                for (var iOutcome = 0; iOutcome < outcomes.length; iOutcome++) {
                    var newOutcome = {
                        outcome: outcomes[iOutcome],
                        conditionForCompletion: conditionON === outcomes[iOutcome] ? conditionValue : ''
                    };
                    $scope.outcomesArray.push(newOutcome);

                }
            } else if ($scope.taskType === 'approval') {
                $scope.outcomeApprovalArray.splice(0, 2);
                for (var iOutcome = 0; iOutcome < outcomes.length; iOutcome++) {
                    var splits = outcomes[iOutcome].split(":");
                    var newOutcome = {
                        outcome: splits[0],
                        customizedValue: splits[1],
                        conditionForCompletion: conditionON === splits[1] ? conditionValue : ''
                    };
                    $scope.outcomeApprovalArray.push(newOutcome);

                }
            }
        }

        function getConditionValues(completionCondition, regEx) {
            if (completionCondition) {
                var match = regEx.exec(completionCondition);
                if (match && match.length >= 2) {
                    return match[1].trim();
                }
            }
            return ''
        }


        // Config for grid
        $scope.outcomesArray = [];

        $scope.outcomeApprovalArray = [
            {
                outcome: 'Approve',
                customizedValue: 'Approve',
                conditionForCompletion: ''
            },
            {
                outcome: 'Reject',
                customizedValue: 'Reject',
                conditionForCompletion: ''
            }
        ]

        if ($scope.taskOutcomesOptionsProperty.value) {
            populateOutcomeOptions();
        }


        function outComeGridRowTemplate() {
            return $timeout(function () {
                return '<div ng-class="{\'ui-grid-row-selected\': row.isSelected, \'ui-grid-row-focused\': row.isFocused}">' +
                    '  <div ng-if="row.entity.new" class="new-row"><input type="text" class="form-control" ng-model="row.entity.outcome" ng-keyup="grid.appScope.addOutcome($event, row)" placeholder="Add new outcome value"/> <div> <span class="ot-icons ot-icon-close" ng-click="grid.appScope.clearOutcome()"></span></div> </div>' +
                    '  <div ng-if="!row.entity.new"  ng-repeat="(colRenderIndex, col) in colContainer.renderedColumns track by col.colDef.name" class="ui-grid-cell" ng-class="{ \'ui-grid-row-header-cell\': col.isRowHeader }"  ui-grid-cell></div>' +
                    '</div>';
            });
        }

        function getEditableCellTemplate(string) {
            return '<workflow-grid-cell-edit ng-model=' + string + ' ></workflow-grid-cell-edit>';
        }

        $scope.getEditRevertMethod = function (value) {
            return function (row) {
                row.entity.outcome = value;
                $scope.$broadcast("uiGridEventEndCellEdit");
            }
        }

        var defaultOutcomeColumnDef = [
            { field: 'rowIndex', displayName: '#', headerCellClass:'gird-header-align-center', maxWidth: 40, cellTemplate: '<span class="workflow-grid-cell">{{rowRenderIndex  + 1}}</span>' },
            { field: 'outcome', displayName: $translate.instant('PROPERTY.OUTCOMES.GRID_TITLE'), enableCellEdit: true, editableCellTemplate: getEditableCellTemplate('row.entity.outcome'), cellTemplate: '<span class="workflow-grid-cell">{{COL_FIELD}} </span>' }
        ];

        function addCompletionConditionToColumnDef(arr, taskType) {
            var isAlreadyAdded = arr[arr.length - 1].field === 'conditionForCompletion';

            if ($scope.isMultiInstance) {
                if (isAlreadyAdded) {
                    return;
                }
                if (taskType === 'default') {
                    arr.push({ field: 'conditionForCompletion', displayName: 'Condition for completion', cellTemplate: '<percentage-input  ng-if="row.isSelected || COL_FIELD" ng-model="row.entity[col.field]" ng-change="grid.appScope.handleConditionChange(row.entity)"></percentage-input>' });
                } else if (taskType === 'approval') {
                    arr.push({ field: 'conditionForCompletion', displayName: 'Condition for completion', cellTemplate: '<percentage-input  ng-if="row.isSelected || COL_FIELD" ng-model="row.entity[col.field]" ng-change="grid.appScope.handleApprovalConditionChange(row.entity)"></percentage-input>' });
                }
            } else {
                if (isAlreadyAdded) {
                    arr.splice(arr.length - 1, 1);
                }
            }
        }

        var approvalOutcomeColumnDef = [{ field: 'rowIndex', displayName: '#', maxWidth: 40, headerCellClass:'gird-header-align-center',cellTemplate: '<span class="workflow-grid-cell">{{rowRenderIndex  + 1}}</span>' },
        { field: 'outcome', displayName: $translate.instant('PROPERTY.OUTCOMES.GRID_TITLE'), cellTemplate: '<span class="workflow-grid-cell">{{COL_FIELD}}</span>' },
        { field: 'customizedValue', displayName: $translate.instant('PROPERTY.OUTCOMES.GRID_CUSTOMIZED_VALUE_TITLE'), enableCellEdit: true, editableCellTemplate: getEditableCellTemplate('row.entity.customizedValue'), cellTemplate: '<span style="display: inline-block;width: 100%;height: 100%;padding-left: 16px;padding-top: 10px;font-size: 14px;color: #000000;">{{row.entity.customizedValue}} </span>' },
        ];


        $scope.$watch('isMultiInstance', function () {
            addCompletionConditionToColumnDef(defaultOutcomeColumnDef, 'default');
            addCompletionConditionToColumnDef(approvalOutcomeColumnDef, 'approval');
        });

        $scope.outcomesGrid = {
            data: $scope.outcomesArray,
            headerRowHeight: 28,
            rowHeight: 40,
            rowTemplate: outComeGridRowTemplate(),
            enableRowSelection: true,
            enableRowHeaderSelection: false,
            multiSelect: false,
            modifierKeysToMultiSelect: false,
            enableHorizontalScrollbar: 0,
            enableColumnMenus: false,
            enableSorting: false,
            enableCellEdit: false,
            columnDefs: defaultOutcomeColumnDef
        };

        $scope.handleConditionChange = function (row) {
            for (var iOutcome = 0; iOutcome < $scope.outcomesArray.length; iOutcome++) {
                var outcome = $scope.outcomesArray[iOutcome];
                if (outcome.outcome !== row.outcome) {
                    outcome.conditionForCompletion = '';
                }
            }
        }

        $scope.handleApprovalConditionChange = function (row) {
            for (var iOutcome = 0; iOutcome < $scope.outcomeApprovalArray.length; iOutcome++) {
                var outcome = $scope.outcomeApprovalArray[iOutcome];
                if (outcome.outcome !== row.outcome) {
                    outcome.conditionForCompletion = '';
                }
            }
        }
        //Grid options for the outcome approval task

        $scope.outcomesApprovalGrid = {
            data: $scope.outcomeApprovalArray,
            headerRowHeight: 28,
            rowHeight: 40,
            enableRowSelection: true,
            enableRowHeaderSelection: false,
            multiSelect: false,
            modifierKeysToMultiSelect: false,
            enableHorizontalScrollbar: 0,
            enableColumnMenus: false,
            enableSorting: false,
            enableCellEdit: false,
            columnDefs: approvalOutcomeColumnDef
        };

        $scope.outcomesGrid.onRegisterApi = function (gridApi) {
            //set gridApi on scope
            $scope.outcomesGridApi = gridApi;
        };

        //Add new outcomes
        $scope.addNewOutcome = function () {
            var gridCanvas = window.rootEle.querySelector('.outcomes-grid .ui-grid-canvas');
            if ($scope.newOutcomeNode && gridCanvas.contains($scope.newOutcomeNode)) {
                $scope.newOutcomeNode.querySelector('input').focus();
                return;
            }
            if ($scope.newOutcomeNode) {
                clearNewOutcomeNode();
            }
            var placeHolder = $translate.instant('TASKOUTCOME.NEWOUTCOME.PLACEHOLDER')
            $scope.newOutcomeNode = angular.element('<div class="new-row"><input type="text" class="form-control" placeholder="' + placeHolder + '"/> <div> <span class="ot-icons ot-icon-close" tabindex="0"></span></div> </div>')[0];
            $scope.newOutcomeNode.querySelector('input').addEventListener('keyup', addOutcome);
            $scope.newOutcomeNode.querySelector('.ot-icon-close').addEventListener('click', clearOutcome);
            $scope.newOutcomeNode.querySelector('.ot-icon-close').addEventListener('keypress', clearOutcome);
            if (gridCanvas.childNodes.length > 1) {
                gridCanvas.insertBefore($scope.newOutcomeNode, gridCanvas.childNodes[0]);
            } else {
                gridCanvas.appendChild($scope.newOutcomeNode);
            }
            $scope.newOutcomeNode.querySelector('input').focus();
            var input  = $scope.newOutcomeNode.querySelector('input');
                input.addEventListener('keydown', function(e) {
                    // Capture the tab keyevent, and make it not do the default behavior.
                    if (e.which === 9) {
                        e.preventDefault();
                        $scope.addOutcome.bind(e);
                    };
                });
        };

        //Add new outcomes keypress handler
        $scope.addOutComeKeyPress = function (event) {
            if (event.keyCode == 13 || event.keyCode == 32) {
                $scope.addNewOutcome();
            }
        }

        function clearNewOutcomeNode() {
            if ($scope.newOutcomeNode) {
                $scope.newOutcomeNode.querySelector('input').removeEventListener('keyup', addOutcome);
                $scope.newOutcomeNode.querySelector('.ot-icon-close').removeEventListener('click', clearOutcome);
                var gridCanvas = window.rootEle.querySelector('.outcomes-grid .ui-grid-canvas');
                if (gridCanvas && gridCanvas.contains($scope.newOutcomeNode)) {
                    gridCanvas.removeChild($scope.newOutcomeNode);
                }
                delete $scope.newOutcomeNode;
            }
        }

        function isValidOutcome(name) {
            if (!name) {
                return;
            }

            return /^[a-zA-Z0-9_]*$/.test(name);
        }

        function addOutcome(event) {
            if (event.keyCode === 13 || event.keyCode === 9) {
                event.preventDefault();
                var outcomeValue = event.target.value;
                if (isValidOutcome(outcomeValue)) {
                    var arr = $scope.outcomesArray.map(function (row) { return row.outcome });
                    if (arr.indexOf(outcomeValue) !== -1) {
                        return;
                    }
                    $timeout(function () {
                        $scope.outcomesArray.push({
                            outcome: outcomeValue,
                            conditionForCompletion: ''
                        });
                    });
                    clearNewOutcomeNode();
                }
            }
        }

        function clearOutcome() {
            clearNewOutcomeNode();
        }

        $scope.$on('$destroy', function () {
            clearNewOutcomeNode();
        });

        // Removed assignment
        $scope.removeOutcome = function () {
            var selectedItems = $scope.outcomesGridApi.selection.getSelectedRows();
            if (selectedItems && selectedItems.length > 0) {
                var index = $scope.outcomesArray.indexOf(selectedItems[0]);
                $scope.outcomesGridApi.selection.toggleRowSelection(selectedItems[0]);
                $scope.outcomesArray.splice(index, 1);

            } else if ($scope.newOutcomeNode) {
                clearNewOutcomeNode();
            }

            $timeout(function () {
                if ($scope.outcomesArray.length > 0) {
                    $scope.outcomesGridApi.selection.toggleRowSelection($scope.outcomesArray[0]);
                }
            });
        };

        //Removed assignment keypress handler
        $scope.removeOutComeKeyPress = function (event) {
            if (event.keyCode == 13 || event.keyCode == 32) {
                $scope.removeOutcome();
            }
        }
    }]);
