/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * String controller
 */

angular.module('workflowModeler').controller('WorkflowStringPropertyCtrl', [ '$scope', function ($scope) {

	$scope.shapeId = $scope.selectedShape.id;
	$scope.valueFlushed = false;
    /** Handler called when input field is blurred */
    $scope.inputBlurred = function() {
    	$scope.valueFlushed = true;
    	if (typeof $scope.property.value === 'string') {
    		$scope.property.value = $scope.property.value.replace(/(<([^>]+)>)/ig,"");
    	}
        $scope.updatePropertyInModel($scope.property);
    };

    $scope.enterPressed = function(keyEvent) {
        // if enter is pressed
        if (keyEvent && keyEvent.which === 13) {
            keyEvent.preventDefault();
            $scope.inputBlurred(); // we want to do the same as if the user would blur the input field
        }
        // else; do nothing
    };
    
    $scope.$on('$destroy', function controllerDestroyed() {
    	if(!$scope.valueFlushed) {
    		if (typeof $scope.property.value === 'string') {
        		$scope.property.value = $scope.property.value.replace(/(<([^>]+)>)/ig,"");
        	}
    		$scope.updatePropertyInModel($scope.property, $scope.shapeId);
    	}
    });

}]);

/*
 * Dropdown controller
 */

angular.module('workflowModeler').controller('WorkflowDropdownPropertyCtrl', [ '$scope','$rootScope', async function ($scope, $rootScope) {
	$scope.shapeId = $scope.selectedShape.id;
    $scope.nameSpaceModel = {};
    $scope.nameSpaces = [];
    $scope.nameSpaces = await window.getNameSpacesFn(); //Load the namespaces from the callback function set in the modeller.
    $scope.$apply(() => {
        if($scope.nameSpaces.length) {
            currentModelValue = $scope.nameSpaces.find(nameSpace => nameSpace.id === $scope.property.value);
            if(currentModelValue) {
               $scope.nameSpaceModel = currentModelValue;
            }
            else if(!currentModelValue && $scope.property.value){
               const invalidSelection = {id: $scope.property.value, display_name: 'Invalid Selection', name: 'Invalid Selection'};
               $scope.nameSpaces.push(invalidSelection);
               $scope.nameSpaceModel = invalidSelection;
            }
               $scope.property.value = $scope.nameSpaceModel.id;
           }
    })

    $scope.onNameSpaceChange = function() {
        $scope.property.value = $scope.nameSpaceModel.id;
    	$scope.updatePropertyInModel($scope.property);    
    }
}]);

/*
 * Boolean controller
 */

angular.module('workflowModeler').controller('WorkflowBooleanPropertyCtrl', ['$scope', function ($scope) {

    $scope.changeValue = function() {
        if ($scope.property.key === 'oryx-defaultflow' && $scope.property.value) {
            var selectedShape = $scope.selectedShape;
            if (selectedShape) {
                var incomingNodes = selectedShape.getIncomingShapes();
                if (incomingNodes && incomingNodes.length > 0) {
                    // get first node, since there can be only one for a sequence flow
                    var rootNode = incomingNodes[0];
                    var flows = rootNode.getOutgoingShapes();
                    if (flows && flows.length > 1) {
                        // in case there are more flows, check if another flow is already defined as default
                        for (var i = 0; i < flows.length; i++) {
                            if (flows[i].resourceId != selectedShape.resourceId) {
                                var defaultFlowProp = flows[i].properties.get('oryx-defaultflow');
                                if (defaultFlowProp) {
                                    flows[i].setProperty('oryx-defaultflow', false, true);
                                }
                            }
                        }
                    }
                }
            }
        }
        $scope.updatePropertyInModel($scope.property);
    };

}]);

/*
 * Text controller
 */

angular.module('workflowModeler').controller('WorkflowTextPropertyCtrl', [ '$scope', '$modal', '$timeout', function($scope, $modal, $timeout) {

    var opts = {
        template:  'editor-app/configuration/properties/text-popup.html',
        scope: $scope,
        prefixEvent: 'textModalEvent'
    };
    
     $scope.$on('textModalEvent.hide.before', function() {
        $timeout(function() {
            $scope.property.mode = 'read';
        }, 0);
    });
    
    // Open the dialog
    _internalCreateModal(opts, $modal, $scope);
}]);

angular.module('workflowModeler').controller('WorkflowTextPropertyPopupCtrl', ['$scope', function($scope) {

    $scope.save = function() {
        $scope.updatePropertyInModel($scope.property);
        $scope.close();
    };
    
    $scope.close = function() {
        $scope.property.mode = 'read';
        $scope.$hide();
    };

    $scope.closeOnKeyPress = function (event) {
        if (event.keyCode == 13 || event.keyCode == 32) {
            $scope.close();
        }
    };
}]);

angular.module('workflowModeler').controller('WorkflowMandatoryInfoPropertyCtrl', ['$scope', '$timeout', function($scope, $timeout) {
    // $timeout(function() {
    //     var text = window.rootEle.querySelector(".mandatoryInfo").innerText;
    //     text = text.split(" ");
    //     var word1 = text.shift();
    //     var word2 = text.shift();
    //     var labelText = word1 + " <span style='color:#df3324'>"+ word2 +"</span> "+text.join(" ");
    //     window.rootEle.querySelector(".mandatoryInfo").innerHTML = labelText;
    // }, 0);
}]);