/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * Input parameters for call activity
 */

angular.module('workflowModeler').controller('WorkflowEventOutParametersCtrl',
    ['$scope', '$modal', '$timeout', '$translate', function ($scope, $modal, $timeout, $translate) {

        // Config for the modal window
        var opts = {
            template: 'editor-app/configuration/properties/event-out-parameters-popup.html',
            scope: $scope
        };

        // Open the dialog
        _internalCreateModal(opts, $modal, $scope);
    }]);

angular.module('workflowModeler').controller('WorkflowEventOutParametersPopupCtrl',
    ['$scope', '$q', '$translate', '$timeout', 'toastService', function ($scope, $q, $translate, $timeout, toastService) {

        // Put json representing form properties on scope
        if ($scope.property.value !== undefined && $scope.property.value !== null
            && $scope.property.value.outParameters !== undefined
            && $scope.property.value.outParameters !== null) {
            // Note that we clone the json object rather then setting it directly,
            // this to cope with the fact that the user can click the cancel button and no changes should have happened
            $scope.parameters = angular.copy($scope.property.value.outParameters);
        } else {
            $scope.parameters = [];
        }

        $scope.translationsRetrieved = false;
        $scope.labels = {};
        $scope.isValid = true;
        $scope.variableNames = {};
        $scope.previousRow;
        var eventNameLabel = $translate('PROPERTY.EVENTOUTPARAMETERS.EVENTNAME');
        var variableNameLabel = $translate('PROPERTY.EVENTOUTPARAMETERS.VARIABLENAME');

        $q.all([eventNameLabel, variableNameLabel]).then(function (results) {
            $scope.labels.eventNameLabel = results[0];
            $scope.labels.variableNameLabel = results[1];
            $scope.translationsRetrieved = true;

            // Config for grid
            $scope.gridOptions = {
                data: $scope.parameters,
                headerRowHeight: 28,
                enableRowSelection: true,
                enableRowHeaderSelection: false,
                multiSelect: false,
                modifierKeysToMultiSelect: false,
                enableHorizontalScrollbar: 0,
                enableColumnMenus: false,
                enableSorting: false,
                columnDefs: [
                    {field: 'eventName', displayName: $scope.labels.eventNameLabel},
                    {field: 'variableName', displayName: $scope.labels.variableNameLabel}]
            };

            $scope.gridOptions.onRegisterApi = function (gridApi) {
                //set gridApi on scope
                $scope.gridApi = gridApi;
                gridApi.selection.on.rowSelectionChanged($scope, function (row) {
                    if(!checkDuplicates()){
                        $scope.gridApi.selection.clearSelectedRows();
                        row = $scope.previousRow;
                        $scope.selectedParameter = row.entity;
                        row.setSelected(true);
                    } else{
                        $scope.selectedParameter = row.entity;
                        $scope.previousRow = row;
                    }
                });
            };
        });

        $scope.isGridDataValid = function () {
            for(var i = 0; i < $scope.parameters.length; i++) {
                var param = $scope.parameters[i];
                if(param.variableName == ''  || param.eventName == '') {
                    $scope.clearValidationAlert = toastService.error($translate.instant('PROPERTY.EVENTPARAMETERS.REQUIRED'));
                    $scope.isValid = false;
                    $scope.variableNames = {};
                    return $scope.isValid; 
                }
                else if($scope.variableNames[param.variableName]) {
                    $scope.clearValidationAlert = toastService.error($translate.instant("PROPERTY.EVENTPARAMETERS.DUPLICATE"));
                    $scope.isValid = false;
                    $scope.variableNames = {};
                    return $scope.isValid;
                }
                else if(!$scope.variableNames[param.variableName]) {
                    $scope.variableNames[param.variableName] = 1;
                }
            }
            $scope.isValid =  true;
            $scope.variableNames = {};
            return $scope.isValid;
        }

        $scope.isCurrentRecordValid = function(event, name) {
            var value = event.target.value;
            if(value == "") {
                $scope.clearValidationAlert = toastService.error($translate.instant('PROPERTY.EVENTPARAMETERS.REQUIRED'));
                $scope.isValid = false;
                event.target.style.borderColor = " red";
                return $scope.isValid;
            } else {
                $scope.isValid = true;
                event.target.style.borderColor = "#757575";
            }
            if(name == "variableNameField") { // checking for duplicates when focus out from variable input.
                checkDuplicates();
            }
        }

        function checkDuplicates() {
            for(var i = 0; i < $scope.parameters.length; i++) {
                var param = $scope.parameters[i];
                if($scope.variableNames[param.variableName]) {
                    $scope.clearValidationAlert = toastService.error($translate.instant("PROPERTY.EVENTPARAMETERS.DUPLICATE"));
                    $scope.isValid = false;
                    $scope.variableNameField.css("borderColor", "red");
                    $scope.variableNames = {};
                    return $scope.isValid;
                }
                else if(!$scope.variableNames[param.variableName]) {
                    $scope.variableNames[param.variableName] = 1;
                    $scope.variableNameField.css("borderColor", "#757575");
                }
            }
            $scope.isValid =  true;
            $scope.variableNames = {};
            return $scope.isValid;
        }

        // Click handler for add button
        $scope.addNewParameter = function () {
            if(!$scope.isGridDataValid()){
                return;
            }
            var newParameter = {
                eventName: '',
                eventType: 'string',
                variableName: ''
            };

            $scope.parameters.push(newParameter);

            $timeout(function () {
                $scope.gridApi.selection.toggleRowSelection(newParameter);
            });
        };

        //Key press handler for add button
        $scope.addParameterKeyPress = function (event) {
            if (event.keyCode == 13) {
                $scope.addNewParameter();
            }
        }

        // Click handler for remove button
        $scope.removeParameter = function () {

            var selectedItems = $scope.gridApi.selection.getSelectedRows();
            if (selectedItems && selectedItems.length > 0) {
                var index = $scope.parameters.indexOf(selectedItems[0]);
                $scope.gridApi.selection.toggleRowSelection(selectedItems[0]);
                $scope.parameters.splice(index, 1);

                if ($scope.parameters.length == 0) {
                    $scope.selectedParameter = undefined;
                }

                $timeout(function () {
                    if ($scope.parameters.length > 0) {
                        $scope.gridApi.selection.toggleRowSelection($scope.parameters[0]);
                    }
                });
                $scope.isGridDataValid();
            }
        };

        //Key press handler for remove button
        $scope.removeParameterKeyPress = function (event) {
            if (event.keyCode == 13) {
                $scope.removeParameter();
            }
        }

        // Click handler for save button
        $scope.save = function () {
            if(!$scope.isGridDataValid()){
                return;
            }

            if($scope.parameters.length)
                $scope.isGridDataValid();
            if(!$scope.isValid){
                return;
            }
            if ($scope.parameters.length > 0) {
                $scope.property.value = {};
                $scope.property.value.outParameters = $scope.parameters;
            } else {
                $scope.property.value = null;
            }

            $scope.updatePropertyInModel($scope.property);
            $scope.close();
        };

        $scope.cancel = function () {
            if ($scope.clearValidationAlert) {
                $scope.clearValidationAlert();
            }
            $scope.close();
        };

        // Close button handler
        $scope.close = function () {
            $scope.property.mode = 'read';
            $scope.$hide();
        };

    }]);