var app = angular.module('workflowModeler');

app.controller('workflowSignatureOperationCntrl', ['$scope', function ($scope) {

    var mandatoryFields = ["oryx-generalgroupheader", 'oryx-overrideid', 'oryx-name', 'oryx-detailsgroupheader', 'oryx-signatureoperation']
    var createContentFields = ['oryx-filefromurl'];
    var sendForSignatureFields = ['oryx-signaturesigners', 'oryx-documenturl'];
    var getDocumentFields = ['oryx-documentid'];
    var deleteDocumentFields = ['oryx-documentid', 'oryx-forcedelete'];
    var getSignatureFields = ['oryx-signatureid'];
    var otherCommonFields = [
        "oryx-responsevariablename",
        "oryx-authenticationdetails",
        "oryx-executiongroupheader",
        "oryx-asynchronousdefinition",
        "oryx-exclusivedefinition",
        "oryx-isforcompensation",
        "oryx-skipexpression",
        "oryx-executionlisteners",
        'oryx-multiintsancegroupheader',
        "oryx-multiinstance_type",
        "oryx-multiinstance_cardinality",
        "oryx-multiinstance_collection",
        "oryx-multiinstance_variable",
        "oryx-multiinstance_condition"
        ]

    if ($scope.property.value) {
        applyFields();
    }

    function applyFields() {
        var propertiesList = [];
        if ($scope.property.value) {
            propertiesList = propertiesList.concat(otherCommonFields);

            switch ($scope.property.value) {
                case 'createdocument':
                    propertiesList = propertiesList.concat(createContentFields);
                    break;
                case 'sendforsignature':
                    propertiesList = propertiesList.concat(sendForSignatureFields);
                    break;
                case 'getdocument':
                    propertiesList = propertiesList.concat(getDocumentFields);
                    break;
                case 'deletedocument':
                    propertiesList = propertiesList.concat(deleteDocumentFields);
                    break;
                case 'getsignature':
                    propertiesList = propertiesList.concat(getSignatureFields);
                    break;
            }
        }
        showAndHideFields(propertiesList)
    }

    $scope.operationChanged = function () {
        applyFields();
        $scope.updatePropertyInModel($scope.property);
    };

    function showAndHideFields(propertiesList) {
        for (var index = 0; index < $scope.selectedItem.properties.length; index++) {
            var prop = $scope.selectedItem.properties[index];
            if (propertiesList.indexOf(prop.key) !== -1) {
                setPropertyHidden(prop, false);
            } else if (mandatoryFields.indexOf(prop.key) === -1) {
                setPropertyHidden(prop, true);
            }
        }
    }

    function setPropertyHidden(property, isHidden) {
        property.hidden = isHidden;
        $scope.updatePropertyInModel(property);
    }
}]);

app.controller('workflowSignatureSignersCfgCntrl', ['$scope', '$modal', function ($scope, $modal) {

    var opts = {
        template: 'editor-app/configuration/properties/signature-signers-cfg-popup.html',
        scope: $scope
    };
    // Open the dialog
    _internalCreateModal(opts, $modal, $scope);
}]);

app.controller('workflowSignersCfgPopupCntrl', ['$scope', '$translate', '$timeout', 'toastService', function ($scope, $translate, $timeout, toastService) {

    var signatureProp = $scope.getPropertyObjectByName('oryx-signaturesigners');
    var subjectProp = $scope.getPropertyObjectByName('oryx-subject');
    var messageProp = $scope.getPropertyObjectByName('oryx-message');

    $scope.model = JSON.parse('{"signers": [], "message": "", "subject": ""}');

    if (subjectProp && subjectProp.value) {
        $scope.model.subject = subjectProp.value;
    }

    if (messageProp && messageProp.value) {
        $scope.model.message = messageProp.value;
    }

    populateSignersArray();

    function populateSignersArray() {
        if (signatureProp && signatureProp.value) {
            var signersArray = signatureProp.value.signers
            if (Array.isArray(signersArray)) {
                for (var iSign = 0; iSign < signersArray.length; iSign++) {
                    var temp = signersArray[iSign];
                    var signer = Object.extend({}, temp);
                    if (signer.approveOnly === true && signer.needsToSign === true) {
                        signer.action = 'approveOnly';
                    } else if (signer.notifyOnly === true) {
                        signer.action = 'notifyOnly';
                    } else {
                        signer.action = 'needsToSign';
                    }
                    delete signer.needsToSign;
                    delete signer.approveOnly;
                    delete signer.notifyOnly;
                    $scope.model.signers.push(signer);
                }
            }
        }
    }

    $scope.tabs = [
        {
            id: 'recipients',
            title: $translate.instant('PROPERTY.SIGNATURE.SIGNERS_CFG.RECIPIENTS_TAB_TITLE')
        },
        {
            id: 'personalize',
            title: $translate.instant('PROPERTY.SIGNATURE.SIGNERS_CFG.PERSONALIZE_TAB_TITLE')
        }]

    function clearErrors() {
        if ($scope.clearValidationAlert) {
            $scope.clearValidationAlert();
        }
    }

    $scope.save = function () {

        clearErrors();

        if ($scope.model.signers.length === 0) {
            $scope.clearValidationAlert = toastService.error($translate.instant('MANDATORY_FIELDS_REQUIRED_VAL_MSG'));
            return;
        }

        if ($scope.model.signers.length > 0) {
            for (var iSigner = 0; iSigner < $scope.model.signers.length; iSigner++) {
                if (!$scope.model.signers[iSigner].email) {
                    $scope.clearValidationAlert = toastService.error($translate.instant('MANDATORY_FIELDS_REQUIRED_VAL_MSG'));
                    return;
                }
            }
        }

        messageProp.value = $scope.model.message;
        subjectProp.value = $scope.model.subject;

        var signers = $scope.model.signers;
        if (signers.length > 0) {
            signatureProp.value = {};
            signatureProp.value.signers = [];
            for (var i = 0; i < signers.length; i++) {
                var signer = Object.extend({}, signers[i]);
                var action = signer.action;
                delete signer.action;
                delete signer.$$hashKey; // this is need to remove unnecessary keys
                switch (action) {
                    case "needsToSign":
                        signer.needsToSign = true;
                        signer.approveOnly = false;
                        signer.notifyOnly = false;
                        break;
                    case "approveOnly":
                        signer.needsToSign = true;
                        signer.approveOnly = true;
                        signer.notifyOnly = false;
                        break;
                    case "notifyOnly":
                        signer.needsToSign = false;
                        signer.approveOnly = false;
                        signer.notifyOnly = true;
                }
                signatureProp.value.signers.push(signer);
            }
        }

        $scope.updatePropertyInModel(signatureProp);
        $scope.updatePropertyInModel(messageProp);
        $scope.updatePropertyInModel(subjectProp);
        $scope.close();
    }

    $scope.cancel = function () {
        $scope.close();
    };

    // Close button handler
    $scope.close = function () {
        clearErrors();
        $scope.property.mode = 'read';
        $scope.$hide();
    };

    var colDef = [
        { field: 'firstName', displayName: $translate.instant('PROPERTY.SIGNATURE.SIGNERS_CFG.FNAME'), cellTemplate: '<span class="workflow-grid-cell">{{COL_FIELD}}</span>' },
        { field: 'lastName', displayName: $translate.instant('PROPERTY.SIGNATURE.SIGNERS_CFG.LNAME'), cellTemplate: '<span class="workflow-grid-cell">{{COL_FIELD}}</span>' },
        { field: 'email', displayName: $translate.instant('PROPERTY.SIGNATURE.SIGNERS_CFG.EMAIL'), cellTemplate: '<span class="workflow-grid-cell">{{COL_FIELD}} </span>' }
    ];

    $scope.signers = [];

    $scope.gridOptions = {
        data: $scope.model.signers,
        headerRowHeight: 28,
        enableRowSelection: true,
        enableRowHeaderSelection: false,
        multiSelect: false,
        modifierKeysToMultiSelect: false,
        enableHorizontalScrollbar: 0,
        enableColumnMenus: false,
        enableSorting: false,
        columnDefs: colDef
    };

    $scope.gridOptions.onRegisterApi = function (gridApi) {
        $scope.signersGridApi = gridApi;

        gridApi.selection.on.rowSelectionChanged($scope, function (row) {
            $scope.selectedSigner = row.entity;
        });

    }

    $scope.addNewSigner = function () {
        var newSigner = {
            "email": "",
            "firstName": "",
            "lastName": "",
            "redirectUrl": "",
            "password": "",
            "action": "needsToSign"
        };
        $scope.model.signers.push(newSigner);

        $timeout(function () {
            $scope.signersGridApi.selection.toggleRowSelection(newSigner);
        });
    }

    $scope.addSignerKeyPress = function (event) {
        if (event.keyCode == 13 || event.keyCode == 32) {
            $scope.addNewSigner();
        }
    }
    $scope.removeSigner = function () {
        var selectedRows = $scope.signersGridApi.selection.getSelectedRows();
        if (selectedRows && selectedRows.length > 0) {
            var index = $scope.model.signers.indexOf(selectedRows[0]);
            $scope.signersGridApi.selection.toggleRowSelection(selectedRows[0]);
            $scope.model.signers.splice(index, 1);
            $scope.selectedSigner = null;
        }


        $timeout(function () {
            if ($scope.model.signers.length > 0) {
                $scope.signersGridApi.selection.toggleRowSelection($scope.model.signers[0]);
            }
        });
    }

    $scope.removeSignerKeyPress = function (event) {
        if (event.keyCode == 13 || event.keyCode == 32) {
            $scope.removeSigner();
        }
    }
}]);

//controllers for the document configuration
app.controller('workflowSigDocCfgWriteCntrl', ['$scope', '$modal', function ($scope, $modal) {

    var opts = {
        template: 'editor-app/configuration/properties/signature-doc-cfg-popup.html',
        scope: $scope
    };
    // Open the dialog
    _internalCreateModal(opts, $modal, $scope);
}]);

app.controller('workflowSigDocCfgPopupCntrl', ['$scope', '$translate', 'toastService', function ($scope, $translate, toastService) {

    var fileFromUrlProp = $scope.getPropertyObjectByName('oryx-filefromurl');

    var docNameProp = $scope.getPropertyObjectByName('oryx-docname');

    var linkExpireProp = $scope.getPropertyObjectByName('oryx-linkexpiredays');
    var autoExpireProp = $scope.getPropertyObjectByName('oryx-autoexpiredays');
    var autoDeleteProp = $scope.getPropertyObjectByName('oryx-autodeletedays');

    var fileNameProp = $scope.getPropertyObjectByName('oryx-filename');
    var fileContentProp = $scope.getPropertyObjectByName('oryx-filecontent');
    var isBase64Prop = $scope.getPropertyObjectByName('oryx-isbase64filecontent');

    $scope.model = {};
    $scope.model.docCreateType = "1";


    if (fileFromUrlProp && fileFromUrlProp.value) {
        $scope.model.documentUrl = fileFromUrlProp.value;
    }

    if (docNameProp && docNameProp.value) {
        $scope.model.docName = docNameProp.value;
    }

    if (linkExpireProp && linkExpireProp.value) {
        $scope.model.linkExpireDays = linkExpireProp.value;
    }

    if (autoExpireProp && autoExpireProp.value) {
        $scope.model.autoExpireDays = autoExpireProp.value;
    }

    if (autoDeleteProp && autoDeleteProp.value) {
        $scope.model.autoDeleteDays = autoDeleteProp.value;
    }

    if (fileNameProp && fileNameProp.value) {
        $scope.model.fileName = fileNameProp.value;
    }

    if (fileContentProp && fileContentProp.value) {
        $scope.model.fileContent = fileContentProp.value;
    }

    if (isBase64Prop && isBase64Prop.value) {
        $scope.model.isBase64 = true;
    }

    if ($scope.model.fileName && $scope.model.fileContent) {
        $scope.model.docCreateType = "2"
    }

    function clearErrors() {
        if ($scope.clearValidationAlert) {
            $scope.clearValidationAlert();
        }
    }

    $scope.save = function () {
        clearErrors();
        if (($scope.model.docCreateType == 1 && !$scope.model.documentUrl)
            || ($scope.model.docCreateType == 2 && (!$scope.model.fileContent || !$scope.model.fileName))) {
            $scope.clearValidationAlert = toastService.error($translate.instant('MANDATORY_FIELDS_REQUIRED_VAL_MSG'));
            return;
        }
        if ($scope.model.docCreateType == 1) {
            fileFromUrlProp.value = $scope.model.documentUrl;
            fileNameProp.value = '';
            fileContentProp.value = '';
            isBase64Prop.value = '';
        } else if ($scope.model.docCreateType == 2) {
            fileNameProp.value = $scope.model.fileName;
            fileContentProp.value = $scope.model.fileContent;
            isBase64Prop.value = $scope.model.isBase64;
            fileFromUrlProp.value = '';
        }

        docNameProp.value = $scope.model.docName;

        linkExpireProp.value = $scope.model.linkExpireDays;
        autoExpireProp.value = $scope.model.autoExpireDays;
        autoDeleteProp.value = $scope.model.autoDeleteDays;

        $scope.updatePropertyInModel(docNameProp);
        $scope.updatePropertyInModel(fileFromUrlProp);
        $scope.updatePropertyInModel(linkExpireProp);
        $scope.updatePropertyInModel(autoExpireProp);
        $scope.updatePropertyInModel(autoDeleteProp);
        $scope.updatePropertyInModel(fileNameProp);
        $scope.updatePropertyInModel(fileContentProp);
        $scope.updatePropertyInModel(isBase64Prop);
        $scope.close();
    }

    $scope.cancel = function () {
        $scope.close();
    }

    $scope.close = function () {
        clearErrors();
        $scope.property.mode = 'read';
        $scope.$hide();
    }
}]);

app.controller('workflowSignersDocCfgDisplayCntrl', ['$scope', function ($scope) {

    var fileFromUrlProp = $scope.getPropertyObjectByName('oryx-filefromurl');
    var fileNameProp = $scope.getPropertyObjectByName('oryx-filename');
    var fileContentProp = $scope.getPropertyObjectByName('oryx-filecontent');

    $scope.isConfigured = false;
    if (fileFromUrlProp.value || (fileNameProp.value && fileContentProp.value)) {
        $scope.isConfigured = true;
    }
}]);