/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * Task listeners
 */

angular.module('workflowModeler').controller('WorkflowTaskListenersCtrl',
    ['$scope', '$modal', '$timeout', '$translate', function ($scope, $modal, $timeout, $translate) {

        // Config for the modal window
        var opts = {
            template: 'editor-app/configuration/properties/task-listeners-popup.html',
            scope: $scope
        };

        // Open the dialog
        _internalCreateModal(opts, $modal, $scope);
    }]);

angular.module('workflowModeler').controller('WorkflowTaskListenersPopupCtrl',
    ['$scope', '$q', '$translate', '$timeout', function ($scope, $q, $translate, $timeout) {

        var scriptClass = 'org.workflow.scripting.secure.listener.WorkflowSecureJavascriptTaskListener';

        // Put json representing form properties on scope
        if ($scope.property.value !== undefined && $scope.property.value !== null
            && $scope.property.value.taskListeners !== undefined
            && $scope.property.value.taskListeners !== null) {

            if ($scope.property.value.taskListeners.constructor == String) {
                $scope.taskListeners = JSON.parse($scope.property.value.taskListeners);
            }
            else {
                // Note that we clone the json object rather then setting it directly,
                // this to cope with the fact that the user can click the cancel button and no changes should have happened
                $scope.taskListeners = angular.copy($scope.property.value.taskListeners);
            }

            for (var i = 0; i < $scope.taskListeners.length; i++) {
                var taskListener = $scope.taskListeners[i];
                var taskListenerClass = taskListener.className;
                if (taskListenerClass !== undefined && taskListenerClass !== '' && taskListenerClass !== scriptClass) {
                    taskListener.implementation = taskListenerClass;
                }
                else if (taskListener.expression !== undefined && taskListener.expression !== '') {
                    taskListener.implementation = taskListener.expression;
                }
                else if (taskListener.delegateExpression !== undefined && taskListener.delegateExpression !== '') {
                    taskListener.implementation = taskListener.delegateExpression;
                }
            }
        } else {
            $scope.taskListeners = [];
        }

        $scope.selectedListener = undefined;
        $scope.selectedField = undefined;
        $scope.fields = [];
        $scope.translationsRetrieved = false;
        $scope.currentView = 'gridView';

        $scope.labels = {};


        var eventPromise = $translate('PROPERTY.TASKLISTENERS.EVENT');
        var implementationPromise = $translate('PROPERTY.TASKLISTENERS.FIELDS.ACTIONS');
        var namePromise = $translate('PROPERTY.TASKLISTENERS.FIELDS.NAME');

        $q.all([eventPromise, implementationPromise, namePromise]).then(function (results) {
            $scope.labels.eventLabel = results[0];
            $scope.labels.actionsLabel = results[1];
            $scope.labels.nameLabel = results[2];
            $scope.translationsRetrieved = true;

            // Config for grid
            $scope.gridOptions = {
                data: $scope.taskListeners,
                headerRowHeight: 28,
                enableRowSelection: true,
                enableRowHeaderSelection: false,
                multiSelect: false,
                modifierKeysToMultiSelect: false,
                enableHorizontalScrollbar: 0,
                enableColumnMenus: false,
                enableSorting: false,
                columnDefs: [{field: 'event', displayName: $scope.labels.eventLabel, maxWidth:180, cellTemplate: getEventTypes()},
                {field: 'actions', displayName: $scope.labels.actionsLabel, cellTemplate: getActionIcons()},
                {field: 'config', displayName: '', maxWidth:60, cellTemplate: '<span ng-click="grid.appScope.openConfigDetails()" class="ot-icons ot-icon-listener_action_settings execution-listener-grid-icons" title="{{\'PROPERTY.LISTENER.CONFIGURE\' | translate}}"></span>'}
            ]
            };

            $scope.gridOptions.onRegisterApi = function (gridApi) {
                //set gridApi on scope
                $scope.gridApi = gridApi;
                gridApi.selection.on.rowSelectionChanged($scope, function (row) {
                    removeInputErrorstyle();
                    $scope.selectedListener = row.entity;
                    $scope.selectedField = undefined;
                    if ($scope.selectedListener) {
                        var fields = $scope.selectedListener.fields;
                        if (fields !== undefined && fields !== null) {
                            for (var i = 0; i < fields.length; i++) {
                                var field = fields[i];
                                if (field.stringValue !== undefined && field.stringValue !== '') {
                                    field.implementation = field.stringValue;
                                } else if (field.expression !== undefined && field.expression !== '') {
                                    field.implementation = field.expression;
                                } else if (field.string !== undefined && field.string !== '') {
                                    field.implementation = field.string;
                                }
                            }
                        } else {
                            $scope.selectedListener.fields = [];
                        }

                        $scope.fields.length = 0;
                        for (var i = 0; i < $scope.selectedListener.fields.length; i++) {
                            $scope.fields.push($scope.selectedListener.fields[i]);
                        }
                    }
                });
            };

            // Config for field grid
            $scope.gridFieldOptions = {
                data: $scope.fields,
                headerRowHeight: 28,
                enableRowSelection: true,
                enableRowHeaderSelection: false,
                multiSelect: false,
                modifierKeysToMultiSelect: false,
                columnDefs: [{field: 'name', displayName: $scope.labels.name},
                    {field: 'actions', displayName: $scope.labels.actionsLabel}]
            };

            $scope.gridFieldOptions.onRegisterApi = function (gridApi) {
                // set gridApi on scope
                $scope.fieldGridApi = gridApi;
                gridApi.selection.on.rowSelectionChanged($scope, function (row) {
                    $scope.selectedField = row.entity;
                });
            };
        });

        $scope.isExpressionConfigured = function (selectedListener) {
            if(selectedListener.expression !== undefined && selectedListener.expression !== ''){
                return true;
            }
            else{
                return false;
            }
        };

        getActionIcons = function(){
            return '<span ng-if="grid.appScope.isExpressionConfigured(row.entity)" class="ot-icons ot-icon-listener_expression_icon execution-listener-grid-icons listener-custom-icon"></span>' +
            '<span ng-if="row.entity.isMailNotification" class="ot-icons ot-icon-listener_email_icon execution-listener-grid-icons listener-custom-icon"></span>' +
            '<span ng-if="row.entity.isExecuteScriptEnabled" class="ot-icons ot-icon-listener_script_icon execution-listener-grid-icons listener-custom-icon"></span>' +
            '<span ng-if="row.entity.publishToEventService" class="ot-icons ot-icon-listener_publish_icon execution-listener-grid-icons"></span>' + 
            '<span ng-if="row.entity.additionalTracing" class="ot-icons ot-icon-listener_tracing execution-listener-grid-icons"></span>' ;  
        }

        getEventTypes = function(){
            return '<select id="eventField" class="form-control listener-row-dropdown" ng-model="row.entity.event">'+
                '<option>create</option><option>assignment</option><option>complete</option></select>';
        }

        $scope.openConfigDetails = function () {
            $scope.currentView = 'configView';
            jQuery(window.rootEle.querySelector("#"+"listenerContainer")).toggleClass('listener-ui-view-change');
        };

        $scope.closeConfigDetails = function () {
            if(!mandatoryFieldsValidation()) {
                return;
            } else {
                // removing previous error color.
                removeInputErrorstyle();
            }          
            $scope.currentView = 'gridView';
            jQuery(window.rootEle.querySelector("#"+"listenerContainer")).toggleClass('listener-ui-view-change');
        };

        $scope.checkForExecuteScript = function () {
            var isScriptEnabled = $scope.selectedListener.isExecuteScriptEnabled;
            var className = $scope.selectedListener.className;
            if (isScriptEnabled && className === '') {
                $scope.selectedListener.className = scriptClass;
            } else if (isScriptEnabled == false && className === scriptClass) {
                $scope.selectedListener.className = '';
            }
        };

        $scope.setFieldsForScriptType = function() {
            var scriptInfo = $scope.selectedListener.scriptInfo;
            var script = (scriptInfo != undefined) ? scriptInfo.script:'';
            if (script !== '') {
                $scope.selectedListener.isExecuteScriptEnabled = true;
            }
            else{
                $scope.selectedListener.isExecuteScriptEnabled = false;
            }
            $scope.setEventFieldData();
        };

        $scope.setFieldsForPublishEvent = function() {
            var eventJsonInfo = $scope.selectedListener.eventInfo;
            var eventJsonData = (eventJsonInfo != undefined) ?eventJsonInfo.eventData :'';
            var eventKey = $scope.selectedListener.eventTypeId;
            if(eventKey !== '' || eventJsonData !== '')
            {
                $scope.selectedListener.publishToEventService = true;
            }
            else{
                $scope.selectedListener.publishToEventService = false;
            }
            $scope.setEventFieldData();
        };

        $scope.setEventFieldData = function() {
            $scope.selectedListener.fields = [];
            var eventJsonInfo = $scope.selectedListener.eventInfo;
            var eventJsonData = (eventJsonInfo != undefined) ?eventJsonInfo.eventData :'';
            var scriptInfo = $scope.selectedListener.scriptInfo;
            var scriptType = (scriptInfo != undefined) ? scriptInfo.scriptType:'';
            var script = (scriptInfo != undefined) ? scriptInfo.script:'';
            var eventTracingJsonInfo = $scope.selectedListener.tracingInfo;
            var eventTracingJsonData = (eventTracingJsonInfo != undefined) ? eventTracingJsonInfo.tracingData :'';
            if (eventJsonData !== '') {
                var eventDataField = {
                    name: 'eventData',
                    implementation: '',
                    stringValue: '',
                    expression: eventJsonData,
                    string: ''
                };
                $scope.selectedListener.fields.push(eventDataField);
            }
            if(eventTracingJsonData !== '') {
                var tracingDataField = {
                    name: 'tracingData',
                    implementation: '',
                    stringValue: '',
                    expression: eventTracingJsonData,
                    string: ''
                };
                $scope.selectedListener.fields.push(tracingDataField);
            }
            if (script !== '') {
                var scriptTypeField = {
                    name: 'language',
                    implementation: scriptType,
                    stringValue: scriptType,
                    expression: '',
                    string: ''
                };
                var scriptField = {
                    name: 'script',
                    implementation: script,
                    stringValue: script,
                    expression: '',
                    string: ''
                };
                $scope.selectedListener.fields.push(scriptTypeField);
                $scope.selectedListener.fields.push(scriptField);
            }
        };

        $scope.setFieldsForMailNotification = function() {
            var mailInfo = $scope.selectedListener.mailInfo;
            var mailInfoKeys = Object.keys(mailInfo);
            for(i=0; i<mailInfoKeys.length; i++)
            {
                if(mailInfo[mailInfoKeys[i]] !== null && mailInfo[mailInfoKeys[i]] !== ''){
                    $scope.selectedListener.isMailNotification = true;
                    break;
                }
                else{
                    $scope.selectedListener.isMailNotification = false;
                }
            }
        };

        $scope.setFieldsForExpression = function() {
            var selectedListenerClass = $scope.selectedListener.className;
            if (selectedListenerClass !== undefined && selectedListenerClass !== '' && selectedListenerClass !== scriptClass) {
                $scope.selectedListener.implementation = selectedListenerClass;
            } else if ($scope.selectedListener.expression !== undefined && $scope.selectedListener.expression !== '') {
                $scope.selectedListener.implementation = $scope.selectedListener.expression;
            } else if ($scope.selectedListener.delegateExpression !== undefined && $scope.selectedListener.delegateExpression !== '') {
                $scope.selectedListener.implementation = $scope.selectedListener.delegateExpression;
            }else {
                $scope.selectedListener.implementation = '';
            }
        };

        $scope.listenerDetailsChanged = function (listenerGroupName) {
            switch (listenerGroupName) {
                case 'expressionGroup':
                    $scope.setFieldsForExpression();
                    break;
                case 'mailGroup':
                    jQuery(window.rootEle.querySelector("#"+"notificationError")).remove();
                    $scope.setFieldsForMailNotification();
                    break;
                case 'scriptGroup':
                    $scope.setFieldsForScriptType();
                    $scope.checkForExecuteScript();
                    break;
                case 'publishListGroup':
                    jQuery(window.rootEle.querySelector("#"+"publishEventError")).remove();
                    $scope.setFieldsForPublishEvent();
                    break;
                case 'additionalTracing':
                    $scope.setFieldsForTracing();
                    break;
            }
        };

        $scope.setFieldsForTracing = function() {
            var eventJsonInfo = $scope.selectedListener.tracingInfo;
            var eventJsonData = (eventJsonInfo != undefined) ? eventJsonInfo.tracingData :'';
            
            if(eventJsonData !== '') 
            {
                $scope.selectedListener.additionalTracing = true;
            }
            else{
                $scope.selectedListener.additionalTracing = false;
            }
            $scope.setEventFieldData();
        }
        // Click handler for add button
        $scope.addNewListener = function () {
            var newListener = {
                event: 'create',
                implementation: '',
                className: '',
                expression: '',
                delegateExpression: '',
                isMailNotification: false,
                isExecuteScriptEnabled: false,
                scriptInfo: {
                    scriptType:'javascript',
                    script:''
                },
                mailInfo:{
                    mailtaskheaders:'',
                    mailtaskto:'',
                    mailtaskfrom:'',
                    mailtaskcc:'',
                    mailtaskbcc:'',
                    mailtasksubject:'',
                    mailtasktext:'',
                    mailtasktextvar: '',
                    mailtaskhtml: '',
                    mailtaskhtmlvar: '',
                    mailtaskcharset: '',
                },
                publishToEventService: false,
                additionalTracing: false,
                eventTypeId:'',
                eventInfo: {
                    eventData:''
                },
                tracingInfo: {
                    tracingData: '',
                },
                fields : []
            };
            $scope.taskListeners.push(newListener);

            $timeout(function () {
                $scope.gridApi.selection.toggleRowSelection(newListener);
            });
        };

        // Click handler for remove button
        $scope.removeListener = function () {
            var selectedItems = $scope.gridApi.selection.getSelectedRows();
            if (selectedItems && selectedItems.length > 0) {
                var index = $scope.taskListeners.indexOf(selectedItems[0]);
                $scope.gridApi.selection.toggleRowSelection(selectedItems[0]);

                $scope.taskListeners.splice(index, 1);

                if ($scope.taskListeners.length == 0) {
                    $scope.selectedListener = undefined;
                }

                $timeout(function () {
                    if ($scope.taskListeners.length > 0) {
                        $scope.gridApi.selection.toggleRowSelection($scope.taskListeners[0]);
                    }
                });
            }
        };

        // Click handler for up button
        $scope.moveListenerUp = function () {
            var selectedItems = $scope.gridApi.selection.getSelectedRows();
            if (selectedItems && selectedItems.length > 0) {
                var index = $scope.taskListeners.indexOf(selectedItems[0]);
                if (index != 0) { // If it's the first, no moving up of course
                    var temp = $scope.taskListeners[index];
                    $scope.taskListeners.splice(index, 1);
                    $timeout(function () {
                        $scope.taskListeners.splice(index + -1, 0, temp);
                        $timeout(function () {
                            $scope.gridApi.selection.toggleRowSelection(temp);
                        });
                    });
                }
            }
        };

        // Click handler for down button
        $scope.moveListenerDown = function () {
            var selectedItems = $scope.gridApi.selection.getSelectedRows();
            if (selectedItems && selectedItems.length > 0) {
                var index = $scope.taskListeners.indexOf(selectedItems[0]);
                if (index != $scope.taskListeners.length - 1) { // If it's the last element, no moving down of course
                    var temp = $scope.taskListeners[index];
                    $scope.taskListeners.splice(index, 1);
                    $timeout(function () {
                        $scope.taskListeners.splice(index + 1, 0, temp);
                        $timeout(function () {
                            $scope.gridApi.selection.toggleRowSelection(temp);
                        });
                    });

                }
            }
        };

        $scope.fieldDetailsChanged = function () {
            if ($scope.selectedField.stringValue != '') {
                $scope.selectedField.implementation = $scope.selectedField.stringValue;
            } else if ($scope.selectedField.expression != '') {
                $scope.selectedField.implementation = $scope.selectedField.expression;
            } else if ($scope.selectedField.string != '') {
                $scope.selectedField.implementation = $scope.selectedField.string;
            } else {
                $scope.selectedField.implementation = '';
            }
        };

        // Click handler for add button
        $scope.addNewField = function () {
            if ($scope.selectedListener) {
                if ($scope.selectedListener.fields == undefined) {
                    $scope.selectedListener.fields = [];
                }

                var newField = {
                    name: 'fieldName',
                    implementation: '',
                    stringValue: '',
                    expression: '',
                    string: ''
                };
                $scope.fields.push(newField);
                $scope.selectedListener.fields.push(newField);

                $timeout(function () {
                    $scope.fieldGridApi.selection.toggleRowSelection(newField);
                });
            }
        };

        // Click handler for remove button
        $scope.removeField = function () {
            var selectedItems = $scope.fieldGridApi.selection.getSelectedRows();
            if (selectedItems && selectedItems.length > 0) {
                var index = $scope.fields.indexOf(selectedItems[0]);
                $scope.fieldGridApi.selection.toggleRowSelection(selectedItems[0]);

                $scope.fields.splice(index, 1);
                $scope.selectedListener.fields.splice(index, 1);

                if ($scope.fields.length == 0) {
                    $scope.selectedField = undefined;
                }

                $timeout(function () {
                    if ($scope.fields.length > 0) {
                        $scope.fieldGridApi.selection.toggleRowSelection($scope.fields[0]);
                    }
                });
            }
        };

        // Click handler for up button
        $scope.moveFieldUp = function () {
            var selectedItems = $scope.fieldGridApi.selection.getSelectedRows();
            if (selectedItems && selectedItems.length > 0) {
                var index = $scope.fields.indexOf(selectedItems[0]);
                if (index != 0) { // If it's the first, no moving up of course
                    var temp = $scope.fields[index];
                    $scope.fields.splice(index, 1);
                    $scope.selectedListener.fields.splice(index, 1);
                    $timeout(function () {
                        $scope.fields.splice(index + -1, 0, temp);
                        $scope.selectedListener.fields.splice(index + -1, 0, temp);
                        $timeout(function () {
                            $scope.fieldGridApi.selection.toggleRowSelection(temp);
                        });
                    });

                }
            }
        };

        // Click handler for down button
        $scope.moveFieldDown = function () {
            var selectedItems = $scope.fieldGridApi.selection.getSelectedRows();
            if (selectedItems && selectedItems.length > 0) {
                var index = $scope.fields.indexOf(selectedItems[0]);
                if (index != $scope.fields.length - 1) { // If it's the last element, no moving down of course
                    var temp = $scope.fields[index];
                    $scope.fields.splice(index, 1);
                    $scope.selectedListeners.fields.splice(index, 1);
                    $timeout(function () {
                        $scope.fields.splice(index + 1, 0, temp);
                        $scope.selectedListener.fields.splice(index + 1, 0, temp);
                        $timeout(function () {
                            $scope.fieldGridApi.selection.toggleRowSelection(temp);
                        });
                    });
                }
            }
        };

        // Click handler for save button
        $scope.save = function () {
            if(mandatoryFieldsValidation()) {
                if ($scope.taskListeners.length > 0) {
                    $scope.property.value = {};
                    $scope.property.value.taskListeners = $scope.taskListeners;
                } else {
                    $scope.property.value = null;
                }

                $scope.updatePropertyInModel($scope.property);
                $scope.close();
            }
        };

        // mail notification and publish listener group data valition for mandatory fields.
        var mandatoryFieldsValidation = function() {
            for(var i = 0; i < $scope.taskListeners.length; i++) {
                var mailGroupFieldStatus = validateEmailGroup($scope.taskListeners[i]);
                var publishListenerFieldStatus = validatePublishListenerGroup($scope.taskListeners[i]);

                if(!mailGroupFieldStatus || !publishListenerFieldStatus)
                {
                    if($scope.clearTimer) {
                        clearTimeout($scope.clearTimer);
                    }
                    $scope.gridApi.selection.toggleRowSelection($scope.taskListeners[i]);
                    jQuery(window.rootEle.querySelector("#validation-alert")).css("display", "table");
                    jQuery(window.rootEle.querySelector("#messageInfo")).text($translate.instant('PROPERTY.LISTENERS.EMAILERRORMESSAGE'));
                    showInputErrorstyle($scope.taskListeners[i]);
                    $scope.clearTimer = setTimeout(function () {
                        jQuery(window.rootEle.querySelector("#validation-alert")).css("display", "none");
                    }, 5000);
                    return false;
                }
            }
            return true;
        };

        var validateEmailGroup = function(taskListener) {
            var mailInfo = taskListener.mailInfo;
            if(taskListener.isMailNotification && 
                (mailInfo.mailtaskfrom == '' || mailInfo.mailtaskto == '' || mailInfo.mailtasktext == ''))
            {
                return false;
            }
            return true;
        };

        var validatePublishListenerGroup = function(taskListener) {
            var eventKey = taskListener.eventTypeId;
            if(taskListener.publishToEventService && (eventKey == undefined || eventKey == ''))
            {
                return false;
            }
            return true;
        };

        var showInputErrorstyle = function(taskListener) {
            var mailInfo = taskListener.mailInfo;
            var eventKey = taskListener.eventTypeId;
            if(taskListener.isMailNotification)
            {
                if (mailInfo.mailtaskto === '') {
                    setErrorStyleToField("mailtaskto");
                }
                if (mailInfo.mailtaskfrom === '') {
                    setErrorStyleToField("mailtaskfrom");
                }
                if (mailInfo.mailtasktext === '') {
                    setErrorStyleToField("mailtasktext");
                }
                if(mailInfo.mailtaskto === '' || mailInfo.mailtaskfrom === '' || mailInfo.mailtasktext === ''){
                    setErrorIconToGroup("notificationHeader","notificationError");
                }
            }
            if (taskListener.publishToEventService && (eventKey == undefined || eventKey === '')) {
                setErrorStyleToField("eventType");
                setErrorIconToGroup("publishEventHeader","publishEventError");
            }
        };

        var validateScriptGroup = function() {
            for(var i = 0; i < $scope.taskListeners.length; i++) {
                var taskListener = $scope.taskListeners[i];
                if(taskListener.isExecuteScriptEnabled)
                {
                    if((taskListener.expression !== undefined && taskListener.expression !== '') || taskListener.isMailNotification){
                        if($scope.clearTimer) {
                            clearTimeout($scope.clearTimer);
                        }
                        $scope.gridApi.selection.toggleRowSelection($scope.taskListeners[i]); 
                        jQuery(window.rootEle.querySelector("#validation-alert")).css("display", "table");
                        jQuery(window.rootEle.querySelector("#messageInfo")).text($translate.instant('PROPERTY.LISTENERS.SCRIPTGROUPVALIDATION'));
                        $scope.clearTimer = setTimeout(function () {
                            jQuery(window.rootEle.querySelector("#validation-alert")).css("display", "none");
                        }, 5000);
                        return false;
                    }
                }
            }
            return true;
        };

        var setErrorStyleToField = function(fieldId) {
            jQuery(window.rootEle.querySelector("#"+fieldId)).css("border", "1px solid red");
        };

        var setErrorIconToGroup = function(groupId,errorId) {
            if(!(jQuery('#'+errorId).length)){
                jQuery(window.rootEle.querySelector("#"+groupId)).after("<span id='"+errorId+"' class='listener-group-error'></span>"); 
            }
        };

        var removeInputErrorstyle = function() {
            removeErrorStyleToField("mailtaskto");
            removeErrorStyleToField("mailtaskfrom");
            removeErrorStyleToField("mailtasktext");
            removeErrorStyleToField("eventType");
            jQuery(window.rootEle.querySelector("#"+"notificationError")).remove();  
            jQuery(window.rootEle.querySelector("#"+"publishEventError")).remove();
        };

        var removeErrorStyleToField = function(fieldId) {
            jQuery(window.rootEle.querySelector("#"+fieldId)).css("border", "1px solid #757575");
        };

        $scope.cancel = function () {
            $scope.close();
        };

        // Close button handler
        $scope.close = function () {
            $scope.property.mode = 'read';
            $scope.$hide();
        };

        $scope.listenerGrpExpanded = function (event, listenerGroup,currentGroup) {
            if (event.keyCode == 39) {
                listenerGroup.expanded = currentGroup;
            } else if (event.keyCode == 37) {
                listenerGroup.expanded = 'false';
            }
        };

    }]);
