/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


WORKFLOW.TOOLBAR = {
    ACTIONS: {

        exportModel: function (services) {

            _internalCreateModal({
                backdrop: true,
                keyboard: true,
                template: 'editor-app/popups/export-model.html',
                scope: services.$scope
            }, services.$modal, services.$scope);
        },

        importProcess: function (services) {
            _internalCreateModal({
                template: 'editor-app/popups/process-import.html',
                scope: services.$scope
            }, services.$modal, services.$scope);
        },

        validate: function (services) {

            _internalCreateModal({
                backdrop: true,
                keyboard: true,
                template: 'editor-app/popups/validate-model.html',
                scope: services.$scope
            }, services.$modal, services.$scope);
        },

        undo: function (services) {

            // Get the last commands
            var lastCommands = services.$scope.undoStack.pop();

            if (lastCommands) {
                // Add the commands to the redo stack
                services.$scope.redoStack.push(lastCommands);

                // Force refresh of selection, might be that the undo command
                // impacts properties in the selected item
                if (services.$rootScope && services.$rootScope.forceSelectionRefresh) {
                    services.$rootScope.forceSelectionRefresh = true;
                }

                // Rollback every command
                for (var i = lastCommands.length - 1; i >= 0; --i) {
                    lastCommands[i].rollback();
                }

                // Update and refresh the canvas
                services.editorManager.handleEvents({
                    type: ORYX.CONFIG.EVENT_UNDO_ROLLBACK,
                    commands: lastCommands
                });

                // Update
                services.editorManager.getCanvas().update();
                services.editorManager.updateSelection();
            }

            var toggleUndo = false;
            if (services.$scope.undoStack.length == 0) {
                toggleUndo = true;
            }

            var toggleRedo = false;
            if (services.$scope.redoStack.length > 0) {
                toggleRedo = true;
            }

            if (toggleUndo || toggleRedo) {
                for (var i = 0; i < services.$scope.items.length; i++) {
                    var item = services.$scope.items[i];
                    if (toggleUndo && item.action === 'WORKFLOW.TOOLBAR.ACTIONS.undo') {
                        services.$scope.safeApply(function () {
                            item.enabled = false;
                        });
                    }
                    else if (toggleRedo && item.action === 'WORKFLOW.TOOLBAR.ACTIONS.redo') {
                        services.$scope.safeApply(function () {
                            item.enabled = true;
                        });
                    }
                }
            }
        },

        redo: function (services) {

            // Get the last commands from the redo stack
            var lastCommands = services.$scope.redoStack.pop();

            if (lastCommands) {
                // Add this commands to the undo stack
                services.$scope.undoStack.push(lastCommands);

                // Force refresh of selection, might be that the redo command
                // impacts properties in the selected item
                if (services.$rootScope && services.$rootScope.forceSelectionRefresh) {
                    services.$rootScope.forceSelectionRefresh = true;
                }

                // Execute those commands
                lastCommands.each(function (command) {
                    command.execute();
                });

                // Update and refresh the canvas
                services.editorManager.handleEvents({
                    type: ORYX.CONFIG.EVENT_UNDO_EXECUTE,
                    commands: lastCommands
                });

                // Update
                services.editorManager.getCanvas().update();
                services.editorManager.updateSelection();
            }

            var toggleUndo = false;
            if (services.$scope.undoStack.length > 0) {
                toggleUndo = true;
            }

            var toggleRedo = false;
            if (services.$scope.redoStack.length == 0) {
                toggleRedo = true;
            }

            if (toggleUndo || toggleRedo) {
                for (var i = 0; i < services.$scope.items.length; i++) {
                    var item = services.$scope.items[i];
                    if (toggleUndo && item.action === 'WORKFLOW.TOOLBAR.ACTIONS.undo') {
                        services.$scope.safeApply(function () {
                            item.enabled = true;
                        });
                    }
                    else if (toggleRedo && item.action === 'WORKFLOW.TOOLBAR.ACTIONS.redo') {
                        services.$scope.safeApply(function () {
                            item.enabled = false;
                        });
                    }
                }
            }
        },

        cut: function (services) {
            WORKFLOW.TOOLBAR.ACTIONS._getOryxEditPlugin(services).editCut();
            for (var i = 0; i < services.$scope.items.length; i++) {
                var item = services.$scope.items[i];
                if (item.action === 'WORKFLOW.TOOLBAR.ACTIONS.paste') {
                    services.$scope.safeApply(function () {
                        item.enabled = true;
                    });
                }
            }
        },

        copy: function (services) {
            WORKFLOW.TOOLBAR.ACTIONS._getOryxEditPlugin(services).editCopy();
            for (var i = 0; i < services.$scope.items.length; i++) {
                var item = services.$scope.items[i];
                if (item.action === 'WORKFLOW.TOOLBAR.ACTIONS.paste') {
                    services.$scope.safeApply(function () {
                        item.enabled = true;
                    });
                }
            }
        },

        paste: function (services) {
            WORKFLOW.TOOLBAR.ACTIONS._getOryxEditPlugin(services).editPaste();
        },

        deleteItem: function (services) {
            WORKFLOW.TOOLBAR.ACTIONS._getOryxEditPlugin(services).editDelete();
        },

        addBendPoint: function (services) {

            var dockerPlugin = WORKFLOW.TOOLBAR.ACTIONS._getOryxDockerPlugin(services);

            var enableAdd = !dockerPlugin.enabledAdd();
            dockerPlugin.setEnableAdd(enableAdd);
            if (enableAdd) {
                dockerPlugin.setEnableRemove(false);
                document.body.style.cursor = 'pointer';
            }
            else {
                document.body.style.cursor = 'default';
            }
        },

        removeBendPoint: function (services) {


            var dockerPlugin = WORKFLOW.TOOLBAR.ACTIONS._getOryxDockerPlugin(services);

            var enableRemove = !dockerPlugin.enabledRemove();
            dockerPlugin.setEnableRemove(enableRemove);
            if (enableRemove) {
                dockerPlugin.setEnableAdd(false);
                document.body.style.cursor = 'pointer';
            }
            else {
                document.body.style.cursor = 'default';
            }
        },

        /**
         * Helper method: fetches the Oryx Edit plugin from the provided scope,
         * if not on the scope, it is created and put on the scope for further use.
         *
         * It's important to reuse the same EditPlugin while the same scope is active,
         * as the clipboard is stored for the whole lifetime of the scope.
         */
        _getOryxEditPlugin: function (services) {
            var $scope = services.$scope;
            var editorManager = services.editorManager;
            if ($scope.oryxEditPlugin === undefined || $scope.oryxEditPlugin === null) {
                $scope.oryxEditPlugin = new ORYX.Plugins.Edit(editorManager.getEditor());
            }
            return $scope.oryxEditPlugin;
        },

        zoomIn: function (services) {
            WORKFLOW.TOOLBAR.ACTIONS._getOryxViewPlugin(services).zoom([1.0 + ORYX.CONFIG.ZOOM_OFFSET]);
        },

        zoomOut: function (services) {
            WORKFLOW.TOOLBAR.ACTIONS._getOryxViewPlugin(services).zoom([1.0 - ORYX.CONFIG.ZOOM_OFFSET]);
        },

        zoomActual: function (services) {
            WORKFLOW.TOOLBAR.ACTIONS._getOryxViewPlugin(services).setAFixZoomLevel(1);
        },

        zoomFit: function (services) {
            WORKFLOW.TOOLBAR.ACTIONS._getOryxViewPlugin(services).zoomFitToModel();
        },

        alignVertical: function (services) {
            WORKFLOW.TOOLBAR.ACTIONS._getOryxArrangmentPlugin(services).alignShapes([ORYX.CONFIG.EDITOR_ALIGN_CENTER]);
        },

        alignHorizontal: function (services) {
            WORKFLOW.TOOLBAR.ACTIONS._getOryxArrangmentPlugin(services).alignShapes([ORYX.CONFIG.EDITOR_ALIGN_MIDDLE]);
        },

        sameSize: function (services) {
            WORKFLOW.TOOLBAR.ACTIONS._getOryxArrangmentPlugin(services).alignShapes([ORYX.CONFIG.EDITOR_ALIGN_SIZE]);
        },



        /**
         * Helper method: fetches the Oryx View plugin from the provided scope,
         * if not on the scope, it is created and put on the scope for further use.
         */
        _getOryxViewPlugin: function (services) {
            var $scope = services.$scope;
            var editorManager = services.editorManager;
            if ($scope.oryxViewPlugin === undefined || $scope.oryxViewPlugin === null) {
                $scope.oryxViewPlugin = new ORYX.Plugins.View(editorManager.getEditor());
            }
            return $scope.oryxViewPlugin;
        },

        _getOryxArrangmentPlugin: function (services) {
            var $scope = services.$scope;
            var editorManager = services.editorManager;
            if ($scope.oryxArrangmentPlugin === undefined || $scope.oryxArrangmentPlugin === null) {
                $scope.oryxArrangmentPlugin = new ORYX.Plugins.Arrangement(editorManager.getEditor());
            }
            return $scope.oryxArrangmentPlugin;
        },

        _getOryxDockerPlugin: function (services) {
            var $scope = services.$scope;
            var editorManager = services.editorManager;
            if ($scope.oryxDockerPlugin === undefined || $scope.oryxDockerPlugin === null) {
                $scope.oryxDockerPlugin = new ORYX.Plugins.AddDocker(editorManager.getEditor());
            }
            return $scope.oryxDockerPlugin;
        }
    }
};


/** Custom controller for the save dialog */
angular.module('workflowModeler').controller('ExportModelCtrl', ['$rootScope', '$scope', '$http', '$location', 'editorManager', '$translate',
    function ($rootScope, $scope, $http, $location, editorManager, $translate) {

        if (editorManager.getCurrentModelId() != editorManager.getModelId()) {
            editorManager.edit(editorManager.getModelId());
        }
        $scope.hideDownloadBtn = false;
        var modelMetaData = editorManager.getBaseModelData();

        var description = '';
        if (modelMetaData.description) {
            description = modelMetaData.description;
        }

        var saveDialog = {
            'name': modelMetaData.name,
            'key': modelMetaData.key,
            'description': description,
            'newVersion': false,
            'comment': ''
        };

        var json = editorManager.getModel();
        delete json.modelId;
        // if process_namespace is an object take extract id from the object
        if(json.properties.process_namespace !== null && typeof json.properties.process_namespace === 'object') {
            json.properties.process_namespace = json.properties.process_namespace.id;
        }
        saveDialog.jsonData = JSON.stringify(json, null, 2);
        var type, modeldata;
        var url = WORKFLOW.URL.getWorkflowConversionService(json.modelId);
        if ($scope.convertToXmlUrl) {
            url = $scope.convertToXmlUrl;
        }
        var headers = $scope.getRootHeaders();
        headers["Content-Type"] = "application/json";
        $http({
            url: url,
            method: 'POST',
            headers: headers,
            data: {
                "modelType": "json",
                "modelContent": json
            }

        }).then(function (response) {
            saveDialog.description = atob(response.data.modelContent);
        }, function (response) {
            $scope.hideDownloadBtn = true;
            console.log(response);
        });

        $scope.saveDialog = saveDialog;

        $scope.status = {
            loading: false
        };

        $scope.close = function () {
            $scope.$hide();
        };
        

        $scope.copyToClopboard = function (activeTab) {
            if (activeTab == "xml") {
                var originalXMLId = jQuery(window.rootEle.querySelector('#xml'));
                var encodedXMLId = jQuery(window.rootEle.querySelector('#xmlEnconded'));
                var encodedXMLData = btoa(encodeURIComponent(originalXMLId.val()).replace(/%([0-9A-F]{2})/g,
                    function toSolidBytes(match, p1) {
                        return String.fromCharCode('0x' + p1);
                    }));
                encodedXMLId.text(encodedXMLData);
                encodedXMLId.select();
                document.execCommand("copy");
                jQuery(window.rootEle.querySelector('#copyToClopboardXML')).text("Copied");
                jQuery(window.rootEle.querySelector('#copyToClopboardXML')).attr("disabled", true);
            } else { // json
                var jsonId = jQuery(window.rootEle.querySelector('#json'));
                jsonId.select();
                document.execCommand("copy");
                jQuery(window.rootEle.querySelector('#copyToClopboardJson')).text("Copied");
                jQuery(window.rootEle.querySelector('#copyToClopboardJson')).attr("disabled", true);
            }
        }

        $scope.tabs = [
            {
                id: 'json',
                title: $translate.instant('EXPORTDIALOG.JSONTABTITLE')
            },
            {
                id: 'xml',
                title: $translate.instant('EXPORTDIALOG.XMLTABTITLE')
            }
        ];

        $scope.downloadXML = function () {
            var element = document.createElement('a');
            element.setAttribute('href',
                'data:text/xml;charset=utf-8,' + encodeURIComponent(saveDialog.description)); 
            element.setAttribute('download', JSON.parse(saveDialog.jsonData).properties.name.replace(/ /g, "") + '.bpmn20.xml');
            document.body.appendChild(element);
            element.click();
            document.body.removeChild(element);
        }

    }]);

angular.module('workflowModeler').controller('ValidateModelCtrl', ['$scope', '$http', 'editorManager',
    function ($scope, $http, editorManager) {

        var editor = editorManager.getEditor();
        var model = editorManager.getModel();

        $scope.status = {
            loading: true
        };

        $scope.model = {
            errors: []
        };

        $scope.cancel = function() {
            $scope.$hide();
            $scope.errorData = "";
        };

        $scope.replacePropertyText = function(description){
            if(description.startsWith("The id of")){
                description =  description.replace("id","name");
            } else if(description.startsWith("The name of")){
                description =  description.replace("name","displayName");
            } else if(description.startsWith("The targetNamespace of")){
                description = description.replace("targetNamespace","namespace");
            }
          return description;
        }
        $scope.closeKeyDown = function (event) {
            if (event.keyCode == 9 && !event.shiftKey) {
                event.preventDefault();
                $scope.errorData = "";
                $scope.modal.$element.find('.ot-icon-close')[0].focus();
            }
        }

        $scope.errorGrid = {
            data: $scope.model.errors,
            headerRowHeight: 28,
            enableRowSelection: true,
            enableRowHeaderSelection: false,
            multiSelect: false,
            modifierKeysToMultiSelect: false,
            enableHorizontalScrollbar: 0,
            enableColumnMenus: false,
            enableSorting: false,
            columnDefs: [
                { field: 'activityName', displayName: 'Name', width: 125 },
                { field: 'defaultDescription', displayName: 'Description', cellTooltip: true},
                { field: 'warning', displayName: 'Critical', cellTemplate: 'editor-app/configuration/properties/errorgrid-critical.html', width: 100 }
            ]
        };

        $scope.errorGrid.onRegisterApi = function (gridApi) {
            //set gridApi on scope
            $scope.gridApi = gridApi;
            gridApi.selection.on.rowSelectionChanged($scope, function (row) {
                if (row.isSelected) {
                    editorManager.navigateTo(row.entity.activityId);
                    $scope.$hide();
                }
            });
        };
        if($scope.errorData) {
          $scope.status.loading = false;
          $scope.errorData.validationErrors.forEach(function (row) {
            row.defaultDescription = $scope.replacePropertyText(row.defaultDescription);
            $scope.model.errors.push(row);
        });
          return;
        }
        var url = WORKFLOW.URL.validateModel();
        if ($scope.validateURL) {
            url = $scope.validateURL;
        }
        var headers = $scope.getRootHeaders();
        headers["Content-Type"] = "application/json;charset=utf-8";
        $http({
            url: url,
            method: 'POST',
            cache: false,
            headers: headers,
            data: {
                "modelType": "json",
                "modelContent": model
            }

        }).then(function (response) {
            $scope.status.loading = false;
            response.data.forEach(function (row) {
                row.defaultDescription = $scope.replacePropertyText(row.defaultDescription);
                $scope.model.errors.push(row);
            });

        }, function (response) {
                $scope.status.loading = false;
                if (response.data.validationErrors) {
                    response.data.validationErrors.forEach(function (row) {
                        row.defaultDescription= $scope.replacePropertyText(row.defaultDescription);
                        $scope.model.errors.push(row);
                    });
                }


        });
    }
]);


angular.module('workflowModeler').controller('ImportProcessModelCtrl', ['$rootScope', '$scope', '$http', 'Upload', '$location', '$timeout', function ($rootScope, $scope, $http, Upload, $location, $timeout) {

    $scope.model = {
        loading: false
    };

    $scope.onFileSelect = function ($files, isIE) {

        for (var i = 0; i < $files.length; i++) {
            var file = $files[i];

            var url;
            if (isIE) {
                url = WORKFLOW.URL.getImportProcessModelTextUrl();
            } else {
                url = WORKFLOW.URL.getWorkflowConversionService();
            }
            if ($scope.importXmlUrl) {
                url = $scope.importXmlUrl;
            }
            var modelData;

            var fileReader = new FileReader();
            fileReader.readAsDataURL(file);
            fileReader.onload = function (e) {
                modelData = e.target.result;
                $scope.importModel(modelData, url);
            }

        }
    };

    $scope.importModel = function (modelData, url) {
        var headers = $scope.getRootHeaders();
        var modelContent = modelData.split("data:text/xml;base64,")[1];
		var validateUrl = WORKFLOW.URL.validateModel();
		if ($scope.validateURL) {
			validateUrl = $scope.validateURL;
		}
        $scope.model.errorMessage = "";
        $http({
			url: validateUrl,
			method: 'POST',
			cache: false,
			headers: headers,
			contentType: 'application/json; charset=utf-8',
			data: {
				"modelType": "xml",
				"modelContent": modelContent
			}
        }).then(function (response) {
                if(response.data.length != 0) {
                    response.data.forEach(function (errorMessage) {
                        if(!$scope.model.errorMessage) {
                            if(errorMessage.validatorSetName =="XML parsing" && errorMessage.problem !=null)
                            {
                                $scope.model.errorMessage = errorMessage.problem;
                            }
                            else{
                                $scope.model.errorMessage = errorMessage.defaultDescription 
                            + ", line no: "+ errorMessage.xmlLineNumber + ", column no: "+ errorMessage.xmlColumnNumber;
                            }
                        } else {
                            $scope.model.errorMessage +=  "\n" + errorMessage.defaultDescription 
                            + ", line no: "+ errorMessage.xmlLineNumber + ", column no: "+ errorMessage.xmlColumnNumber;
                        }
                        $scope.model.error = true;
                        $scope.model.loading = false;
                    });
                } else if (response.data.length == 0) {
                    $http({
                        url: url,
                        method: 'POST',
                        headers: headers,
                        contentType: 'application/json; charset=utf-8',
                        dataType: "json",
                        data: {
                            "modelType": "xml",
                            "modelContent": modelContent
                        }
                    }).then(function (data) {
                        $scope.model.loading = false;
                        $rootScope.editorInitialized = false;
                        $rootScope.importedModel = data;
                        if (window.document.title.indexOf('*') == 0) {
                            window.document.title = window.document.title.slice(1);
                        }
                        $scope.cancel();
                    }, function (data) {
                        if (data && data.message) {
                            $scope.model.errorMessage = data.message;
                        }
                        $scope.model.error = true;
                        $scope.model.loading = false;
                    });
                }
        }, function (response) {
            if (response && response.message) {
                $scope.model.errorMessage = response.message;
            } else if (response && response.data && response.data.message) {
                $scope.model.errorMessage = response.data.message;
            }
            $scope.model.error = true;
            $scope.model.loading = false;
        });
    }

    $scope.onFileSelectKeyPress = function (event) {
        event.target.click();
    }

    $scope.fileSelectKeyDown = function (event) {
        if (event.keyCode == 9 && !event.shiftKey) {
            event.preventDefault();
            $scope.modal.$element.find('.ot-icon-close')[0].focus();
        }
    };

    $scope.cancel = function () {
        if (!$scope.model.loading) {
            $scope.$hide();
        }
    };
}]);