/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
'use strict';
/**
 * General bootstrap of the application.
 */
var stencilJson = require('../stencilset_bpmn.json');
angular.module('workflowModeler')
    .controller('EditorController', ['$rootScope', '$scope', '$http', '$q', '$timeout', '$window', '$location', '$translate', '$modal', 'editorManager', 'FormBuilderService',
        function ($rootScope, $scope, $http, $q, $timeout, $window, $location, $translate, $modal, editorManager, FormBuilderService) {

            $rootScope.editorFactory = $q.defer();

            $rootScope.forceSelectionRefresh = false;

            $rootScope.ignoreChanges = false; // by default never ignore changes

            $rootScope.validationErrors = [];

            $rootScope.staticIncludeVersion = Date.now();

            const KEY_ENTER = "13";

            /**
             * Initialize the event bus: couple all Oryx events with a dispatch of the
             * event of the event bus. This way, it gets much easier to attach custom logic
             * to any event.
             */

            function initScrollHandling() {
                var canvasSection = jQuery(window && window.rootEle && window.rootEle.querySelector('#canvasSection'));
                canvasSection.scroll(function () {

                    // Hides the resizer and quick menu items during scrolling

                    var selectedElements = editorManager.getSelection();
                    var subSelectionElements = editorManager.getSubSelection();

                    $scope.selectedElements = selectedElements;
                    $scope.subSelectionElements = subSelectionElements;
                    if (selectedElements && selectedElements.length > 0) {
                        $rootScope.selectedElementBeforeScrolling = selectedElements[0];
                    }

                    window && window.rootEle && (window.rootEle.querySelectorAll('.quick-menu-items-container')[0].style.display = 'none');
                    jQuery(window.rootEle.querySelectorAll('.Oryx_button')).each(function (i, obj) {
                        $scope.orginalOryxButtonStyle = obj.style.display;
                        obj.style.display = 'none';
                    });
                    jQuery(window.rootEle.querySelectorAll('.resizer_southeast')).each(function (i, obj) {
                        $scope.orginalResizerSEStyle = obj.style.display;
                        obj.style.display = 'none';
                    });
                    jQuery(window.rootEle.querySelectorAll('.resizer_northwest')).each(function (i, obj) {
                        $scope.orginalResizerNWStyle = obj.style.display;
                        obj.style.display = 'none';
                    });
                    editorManager.handleEvents({
                        type: ORYX.CONFIG.EVENT_CANVAS_SCROLL
                    });
                });

                canvasSection.scrollStopped(function () {

                    // Puts the quick menu items and resizer back when scroll is stopped.

                    editorManager.setSelection([]); // needed cause it checks for element changes and does nothing if the elements are the same
                    editorManager.setSelection($scope.selectedElements, $scope.subSelectionElements);
                    $scope.selectedElements = undefined;
                    $scope.subSelectionElements = undefined;

                    function handleDisplayProperty(obj) {
                        if (jQuery(obj).position().top > 0) {
                            obj.style.display = 'block';
                        } else {
                            obj.style.display = 'none';
                        }
                    }

                    jQuery(window.rootEle.querySelectorAll('.Oryx_button')).each(function (i, obj) {
                        if (obj.style.display != 'block') {
                            obj.style.display = 'none';
                        }
                    });
                    jQuery(window.rootEle.querySelectorAll('.resizer_southeast')).each(function (i, obj) {
                        handleDisplayProperty(obj);
                    });
                    jQuery(window.rootEle.querySelectorAll('.resizer_northwest')).each(function (i, obj) {
                        handleDisplayProperty(obj);
                    });

                });
            }

            /**
             * Initialize the Oryx Editor when the content has been loaded
             */
            window.setTimeout(function () {
                if (!$rootScope.editorInitialized) {
                    $rootScope.openHamburgMenu = true;
                    var paletteHelpWrapper = jQuery(window && window.rootEle && window.rootEle.querySelector('#paletteHelpWrapper'));
                    var paletteHelpWrapperInitWidth = paletteHelpWrapper.width();
                    var hamburgerMenu = jQuery(window && window.rootEle && window.rootEle.querySelector('#hamburgerMenu'));
                    var paletteSectionOpen = jQuery(window && window.rootEle && window.rootEle.querySelector('#paletteSectionOpen'));
                    var propSectionOpen = jQuery(window && window.rootEle && window.rootEle.querySelector('#propSectionOpen'));
                    var propertiesHelpWrapper = jQuery(window && window.rootEle && window.rootEle.querySelector('#propertiesHelpWrapper'));
                    var contentCanvasWrapper = jQuery(window && window.rootEle && window.rootEle.querySelector('#contentCanvasWrapper'));
                    var propsWindowTitle = jQuery(window && window.rootEle && window.rootEle.querySelector('#propsWindowTitle'));
                    var hamburgerMenuImage = jQuery(window && window.rootEle && window.rootEle.querySelector('#hamburgerMenuImage'));
                    var stencilDragbarHandler = jQuery(window && window.rootEle && window.rootEle.querySelector('#stencilDragbar'));
                    var rootDiv = jQuery(window && window.rootEle && window.rootEle.querySelector('#modeler'));
                    var propertiesWrapper = jQuery(window && window.rootEle && window.rootEle.querySelector('#propertiesHelpWrapper'));
                    var propertySection = jQuery(window && window.rootEle && window.rootEle.querySelector('#propertySection'));
                    var rightPadding = 0;
                    var leftPadding = 0;
                    var editorToolbar = jQuery(window && window.rootEle && window.rootEle.querySelector('#editor-toolbar'));
                    $scope.menuOpen = true
                    $scope.toggleSidePanels = false

                    var toggleSidePanels = jQuery(window && window.rootEle && window.rootEle.querySelector('#editor-toggle-button'));
                    var propSectionIcon = jQuery(window && window.rootEle && window.rootEle.querySelector('#propSectionOpen'));
                    var processNavigator = jQuery(window && window.rootEle && window.rootEle.querySelector('#process-navigator-container'));

                    if(!$rootScope.showSidePanels) {
                        paletteHelpWrapper.addClass('close');
                        contentCanvasWrapper.addClass('collapsedCanvasWrapper');
                        paletteSectionOpen.removeClass('hidden');
                        leftPadding = 42;
                        $scope.menuOpen = false;
                        processNavigator.css({"display": "none"});

                        propSectionIcon.click();
                    } else {
                        processNavigator.css({"display": "block"});
                    }

                    toggleSidePanels.on('click', function () {
                        leftPadding = 0;
                        paletteSectionOpen.addClass('hidden');
                        if ($scope.toggleSidePanels) { // open side panels
                            hamburgerMenu.addClass("side-panels-opened");
                            hamburgerMenuImage.addClass("side-panels-opened");
                            processNavigator.css({"display": "block"});
                            hamburgerMenu.removeClass("side-panels-closed");
                            hamburgerMenuImage.removeClass("side-panels-closed");
                            paletteHelpWrapper.removeClass('close');
                            contentCanvasWrapper.css("width","82%");
                            paletteHelpWrapper.css("width","18%");
                            stencilDragbarHandler.removeClass("hidden");
                            stencilDragbarHandler.css("left", "18%");
                            contentCanvasWrapper.removeClass('collapsedCanvasWrapper');
                            if (contentCanvasWrapper.hasClass("collapsed-property-window")) {
                                propSectionIcon.click();
                            }
                            $timeout(function () {
                                $window.dispatchEvent(new Event("resize"));
                            }, 100);
                            $scope.toggleSidePanels = false;
                            $scope.menuOpen = true;
                        } else { // close side panels
                            hamburgerMenu.addClass("side-panels-closed");
                            hamburgerMenuImage.addClass("side-panels-closed");
                            hamburgerMenu.removeClass("side-panels-opened");
                            hamburgerMenuImage.removeClass("side-panels-opened");
                            paletteHelpWrapper.addClass('close');
                            contentCanvasWrapper.addClass('collapsedCanvasWrapper');
                            stencilDragbarHandler.addClass("hidden");
                            contentCanvasWrapper.css("width","100%");
                            if (!contentCanvasWrapper.hasClass("collapsed-property-window")) {
                                propSectionIcon.click();
                            }
                            $timeout(function () {
                                $window.dispatchEvent(new Event("resize"));
                            }, 100);
                            $scope.toggleSidePanels = true;
                            $scope.menuOpen = false;
                        }
                    });
                    onResize();

                    var dragging = false;
                    
                    stencilDragbarHandler.mousedown(function(e){
                        e.preventDefault();
                        dragging = true;
                        stencilDragbarHandler.css("background-color","#232E72");
                        rootDiv.mousemove(function(e) {                    
                            stencilDragbarHandler.css("left",e.pageX+2);
                        });
                    });

                    rootDiv.mouseup(function(e){
                        if (dragging) 
                        {
                            paletteHelpWrapper.css("width",e.pageX+2);
                            rootDiv.unbind('mousemove');
                            dragging = false;
                            if(e.pageX+2 <= 60) {
                                paletteHelpWrapper.addClass('close');
                                contentCanvasWrapper.addClass('collapsedCanvasWrapper');
                                paletteSectionOpen.removeClass('hidden');
                                leftPadding = 42;
                                $scope.menuOpen = false;
                                processNavigator.css({"display": "none"});
                                stencilDragbarHandler.addClass("hidden");
                            }
                            stencilDragbarHandler.css("background-color","transparent");
                            contentCanvasWrapper.width(contentCanvasWrapper.width() + paletteHelpWrapperInitWidth - paletteHelpWrapper.width());
                            paletteHelpWrapperInitWidth = paletteHelpWrapper.width();
                            $timeout(function () {
                                $window.dispatchEvent(new Event("resize"));
                            }, 10);
                        }
                    });

                    hamburgerMenu.on('click', function () {
                        hamburgerMenu.addClass("side-panels-opened");
                        hamburgerMenuImage.addClass("side-panels-opened");
                        if (paletteSectionOpen.hasClass('close')) {
                            paletteSectionOpen.removeClass('close');
                        }
                        if ($scope.menuOpen) {
                            paletteHelpWrapper.addClass('close');
                            contentCanvasWrapper.addClass('collapsedCanvasWrapper');
                            paletteSectionOpen.removeClass('hidden');
                            leftPadding = 42;
                            contentCanvasWrapper.css("width","100%");
                            $scope.menuOpen = false;
                            stencilDragbarHandler.addClass("hidden");
                            $timeout(function () {
                                $window.dispatchEvent(new Event("resize"));
                            }, 100);
                        } else {
                            processNavigator.css({"display": "block"});
                            paletteHelpWrapper.removeClass('close');
                            contentCanvasWrapper.removeClass('collapsedCanvasWrapper');
                            paletteSectionOpen.addClass('hidden');
                            leftPadding = 0;
                            $scope.menuOpen = true;
                            contentCanvasWrapper.css("width","82%");
                            paletteHelpWrapper.css("width","18%");
                            stencilDragbarHandler.removeClass("hidden");
                            stencilDragbarHandler.css("left", "18%");
                            paletteHelpWrapperInitWidth = paletteHelpWrapper.width();
                            $timeout(function () {
                                $window.dispatchEvent(new Event("resize"));
                            }, 100);
                        }
                    });

                    hamburgerMenu.on('keydown', function (event) {
                        if (event.keyCode == KEY_ENTER) {
                            hamburgerMenu.click();
                        }
                    });

                    propSectionOpen.on('click', function () {
                        // propertiesHelpWrapper.removeClass('close');
                        // jQuery('#propertySection').removeClass('collapsed');
                        // propSectionOpen.addClass('hidden');
                        contentCanvasWrapper.addClass('properties-helper-opened');
                        rightPadding = 0;
                        $timeout(function () {
                            $window.dispatchEvent(new Event("resize"));
                        }, 100);
                    });

                    // propsWindowTitle.on('click', function () {
                    //     // propertiesHelpWrapper.addClass('close');
                    //     // propSectionOpen.removeClass('hidden');
                    //     // rightPadding = 45;
                    //     // $timeout(function () {
                    //     //     $window.dispatchEvent(new Event("resize"));
                    //     // }, 100);
                    // });

                    /**
                     * A 'safer' apply that avoids concurrent updates (which $apply allows).
                     */
                    $rootScope.safeApply = function (fn) {
                        if (this.$root) {
                            var phase = this.$root.$$phase;
                            if (phase == '$apply' || phase == '$digest') {
                                if (fn && (typeof (fn) === 'function')) {
                                    fn();
                                }
                            } else {
                                this.$apply(fn);
                            }

                        } else {
                            this.$apply(fn);
                        }
                    };

                    $rootScope.addHistoryItem = function (resourceId) {
                        var modelMetaData = editorManager.getBaseModelData();

                        var historyItem = {
                            id: modelMetaData.modelId,
                            name: modelMetaData.name,
                            key: modelMetaData.key,
                            stepId: resourceId,
                            type: 'bpmnmodel'
                        };

                        if (editorManager.getCurrentModelId() != editorManager.getModelId()) {
                            historyItem.subProcessId = editorManager.getCurrentModelId();
                        }

                        $rootScope.editorHistory.push(historyItem);
                    };

                    $rootScope.getStencilSetName = function () {
                        var modelMetaData = editorManager.getBaseModelData();
                        if (modelMetaData.model.stencilset.namespace == 'http://b3mn.org/stencilset/cmmn1.1#') {
                            return 'cmmn1.1';
                        } else {
                            return 'bpmn2.0';
                        }
                    };

                    /**
                     * Initialize the event bus: couple all Oryx events with a dispatch of the
                     * event of the event bus. This way, it gets much easier to attach custom logic
                     * to any event.
                     */

                    $rootScope.editorFactory.promise.then(function () {

                        $rootScope.formItems = undefined;

                        WORKFLOW.eventBus.editor = $rootScope.editor;

                        var eventMappings = [{
                                oryxType: ORYX.CONFIG.EVENT_SELECTION_CHANGED,
                                WorkflowType: WORKFLOW.eventBus.EVENT_TYPE_SELECTION_CHANGE
                            },
                            {
                                oryxType: ORYX.CONFIG.EVENT_DBLCLICK,
                                WorkflowType: WORKFLOW.eventBus.EVENT_TYPE_DOUBLE_CLICK
                            },
                            {
                                oryxType: ORYX.CONFIG.EVENT_MOUSEOUT,
                                WorkflowType: WORKFLOW.eventBus.EVENT_TYPE_MOUSE_OUT
                            },
                            {
                                oryxType: ORYX.CONFIG.EVENT_MOUSEOVER,
                                WorkflowType: WORKFLOW.eventBus.EVENT_TYPE_MOUSE_OVER
                            },
                            {
                                oryxType: ORYX.CONFIG.EVENT_EDITOR_INIT_COMPLETED,
                                WorkflowType: WORKFLOW.eventBus.EVENT_TYPE_EDITOR_READY
                            },
                            {
                                oryxType: ORYX.CONFIG.EVENT_PROPERTY_CHANGED,
                                WorkflowType: WORKFLOW.eventBus.EVENT_TYPE_PROPERTY_VALUE_CHANGED
                            }

                        ];

                        eventMappings.forEach(function (eventMapping) {
                            editorManager.registerOnEvent(eventMapping.oryxType, function (event) {
                                WORKFLOW.eventBus.dispatch(eventMapping.WorkflowType, event);
                            });
                        });

                    });

                    // Hook in resizing of main panels when window resizes
                    // TODO: perhaps move to a separate JS-file?
                    $scope.$on('$destroy', function () {
                        window.removeEventListener('resize', onResize)
                    });

                    window.addEventListener('resize', onResize);

                    function onResize() {

                        // Calculate the offset based on the bottom of the module header
                        var offset = jQuery(window && window.rootEle && window.rootEle.querySelector("#editor-toolbar")).offset();
                        var propSectionHeight = 0;
                        var propSection = jQuery(window && window.rootEle && window.rootEle.querySelector('#propertySection'));
                        var canvas = jQuery(window && window.rootEle && window.rootEle.querySelector('#canvasSection'));
                        var mainHeader = jQuery(window && window.rootEle && window.rootEle.querySelector('#main-header'));
                        var hamburgerMenu = jQuery(window && window.rootEle && window.rootEle.querySelector('#hamburgerMenu'));
                        var canvasHelpWrapper = jQuery(window && window.rootEle && window.rootEle.querySelector('#canvasHelpWrapper'));
                        var hamburegerMenuImg = jQuery(window && window.rootEle && window.rootEle.querySelector('#hamburgerMenuImage'));
                        var toolbarHeight = jQuery(window && window.rootEle && window.rootEle.querySelector('#editor-toolbar')).height();


                        if (offset == undefined || offset === null ||
                            propSectionHeight === undefined || propSectionHeight === null ||
                            canvas === undefined || canvas === null || mainHeader === null) {
                            return;
                        }

                        if ($rootScope.editor) {
                            var selectedElements = editorManager.getSelection();
                            var subSelectionElements = editorManager.getSelection();

                            $scope.selectedElements = selectedElements;
                            $scope.subSelectionElements = subSelectionElements;
                            if (selectedElements && selectedElements.length > 0) {
                                $rootScope.selectedElementBeforeScrolling = selectedElements[0];

                                editorManager.setSelection([]); // needed cause it checks for element changes and does nothing if the elements are the same
                                editorManager.setSelection($scope.selectedElements, $scope.subSelectionElements);
                                $scope.selectedElements = undefined;
                                $scope.subSelectionElements = undefined;
                            }
                        }

                        hamburegerMenuImg.height(toolbarHeight);
                        hamburgerMenu.height(toolbarHeight);
                        var totalAvailableHeight = $(window.rootEle.getRootNode().host.parentElement).offsetHeight - 2 - (mainHeader.height() ? mainHeader.height() : 0) - hamburgerMenu.height();
                        var totalAvailableWidth = window.rootEle.children[0].offsetWidth - propertiesHelpWrapper.width() - paletteHelpWrapper.width() - leftPadding;
                        canvas.height(totalAvailableHeight - propSectionHeight);
                        canvas.width(totalAvailableWidth);
                        canvasHelpWrapper.width(totalAvailableWidth);
                        canvasHelpWrapper.css({
                            "left": leftPadding
                        });
                        contentCanvasWrapper.height(totalAvailableHeight);
                        propertiesHelpWrapper.height(totalAvailableHeight);
                        //editorToolbar.width(window.rootEle.children[0].offsetWidth-47);  // width: hamburger menu 42 + padding 5
                        propSectionOpen.css("right", propertiesHelpWrapper.width());
                        var footerHeight = jQuery(window && window.rootEle && window.rootEle.querySelector('#paletteSectionFooter')).height();
                        var treeViewHeight = jQuery(window && window.rootEle && window.rootEle.querySelector('#process-navigator-container')).height();
                        jQuery(window && window.rootEle && window.rootEle.querySelector('#paletteSection')).height(totalAvailableHeight - treeViewHeight);

                        // Update positions of the resize-markers, according to the canvas

                        var actualCanvas = null;
                        if (canvas && canvas[0].children[1]) {
                            actualCanvas = canvas[0].children[1];
                        }

                        var canvasTop = canvas.position().top;
                        var canvasLeft = canvas.position().left;
                        var canvasHeight = canvas[0].clientHeight;
                        var canvasWidth = canvas[0].clientWidth;
                        var iconCenterOffset = 8;
                        var widthDiff = 0;

                        var actualWidth = 0;
                        if (actualCanvas) {
                            // In some browsers, the SVG-element clientwidth isn't available, so we revert to the parent
                            if (actualCanvas.hasClassName('Oryx_button')) {
                                actualWidth = actualCanvas.parentNode.clientWidth;
                            } else {
                                actualWidth = actualCanvas.clientWidth || actualCanvas.parentNode.clientWidth;
                            }
                        }

                        if (actualWidth < canvas[0].clientWidth) {
                            widthDiff = actualWidth - canvas[0].clientWidth;
                            // In case the canvas is smaller than the actual viewport, the resizers should be moved
                            canvasLeft -= widthDiff / 2;
                            canvasWidth += widthDiff;
                        }

                        var iconWidth = 17;
                        var iconOffset = 20;

                        var north = jQuery(window && window.rootEle && window.rootEle.querySelector('#canvas-grow-N'));
                        north.css('top', canvasTop + iconOffset + 'px');
                        north.css('left', canvasLeft - 10 + (canvasWidth - iconWidth) / 2 + 'px');

                        var south = jQuery(window && window.rootEle && window.rootEle.querySelector('#canvas-grow-S'));
                        south.css('top', (canvasTop + canvasHeight - iconOffset - iconCenterOffset) + 'px');
                        south.css('left', canvasLeft - 10 + (canvasWidth - iconWidth) / 2 + 'px');

                        var east = jQuery(window && window.rootEle && window.rootEle.querySelector('#canvas-grow-E'));
                        east.css('top', canvasTop - 10 + (canvasHeight - iconWidth) / 2 + 'px');
                        east.css('left', (canvasLeft + canvasWidth - iconOffset - iconCenterOffset) + 'px');

                        var west = jQuery(window && window.rootEle && window.rootEle.querySelector('#canvas-grow-W'));
                        west.css('top', canvasTop - 10 + (canvasHeight - iconWidth) / 2 + 'px');
                        west.css('left', canvasLeft + iconOffset + 'px');

                        north = jQuery(window && window.rootEle && window.rootEle.querySelector('#canvas-shrink-N'));
                        north.css('top', canvasTop + iconOffset + 'px');
                        north.css('left', canvasLeft + 10 + (canvasWidth - iconWidth) / 2 + 'px');

                        south = jQuery(window && window.rootEle && window.rootEle.querySelector('#canvas-shrink-S'));
                        south.css('top', (canvasTop + canvasHeight - iconOffset - iconCenterOffset) + 'px');
                        south.css('left', canvasLeft + 10 + (canvasWidth - iconWidth) / 2 + 'px');

                        east = jQuery(window && window.rootEle && window.rootEle.querySelector('#canvas-shrink-E'));
                        east.css('top', canvasTop + 10 + (canvasHeight - iconWidth) / 2 + 'px');
                        east.css('left', (canvasLeft + canvasWidth - iconOffset - iconCenterOffset) + 'px');

                        west = jQuery(window && window.rootEle && window.rootEle.querySelector('#canvas-shrink-W'));
                        west.css('top', canvasTop + 10 + (canvasHeight - iconWidth) / 2 + 'px');
                        west.css('left', canvasLeft + iconOffset + 'px');
                    };

                    jQuery(window).trigger('resize');

                    jQuery.fn.scrollStopped = function (callback) {
                        jQuery(this).scroll(function () {
                            var self = this,
                                $this = jQuery(self);
                            if ($this.data('scrollTimeout')) {
                                clearTimeout($this.data('scrollTimeout'));
                            }
                            $this.data('scrollTimeout', setTimeout(callback, 50, self));
                        });
                    };

                    WORKFLOW.eventBus.addListener('ORYX-EDITOR-LOADED', function () {
                        this.editorFactory.resolve();
                        this.editorInitialized = true;
                        this.modelData = editorManager.getBaseModelData();
                        if (window.rootEle) {
                            var event = new CustomEvent('modelerLoaded');
                            window.rootEle.dispatchEvent(event);
                        }
                        
                    }, $rootScope);

                    WORKFLOW.eventBus.addListener(WORKFLOW.eventBus.EVENT_TYPE_EDITOR_READY, function () {
                        var url = window.location.href;
                        var regex = new RegExp("[?&]subProcessId(=([^&#]*)|&|#|$)");
                        var results = regex.exec(url);
                        if (results && results[2]) {
                            editorManager.edit(decodeURIComponent(results[2].replace(/\+/g, " ")));
                        }
                    });
                }
                initScrollHandling();
            });

            $scope.$on('$locationChangeStart', function (event, next, current) {
                if ($rootScope.editor && !$rootScope.ignoreChanges) {
                    var plugins = $rootScope.editor.loadedPlugins;

                    var savePlugin;
                    for (var i = 0; i < plugins.length; i++) {
                        if (plugins[i].type == 'ORYX.Plugins.Save') {
                            savePlugin = plugins[i];
                            break;
                        }
                    }

                    if (savePlugin && savePlugin.hasChanges()) {
                        // Always prevent location from changing. We'll use a popup to determine the action we want to take
                        event.preventDefault();

                        if (!$scope.unsavedChangesModalInstance) {

                            var handleResponseFunction = function (discard) {
                                $scope.unsavedChangesModalInstance = undefined;
                                if (discard) {
                                    $rootScope.ignoreChanges = true;
                                    $location.url(next.substring(next.indexOf('/#') + 2));
                                } else {
                                    $rootScope.ignoreChanges = false;
                                    $rootScope.setMainPageById('processes');
                                }
                            };

                            $scope.handleResponseFunction = handleResponseFunction;

                            _internalCreateModal({
                                template: 'editor-app/popups/unsaved-changes.html',
                                scope: $scope
                            }, $modal, $scope);
                        }
                    }
                }
            });

            // Always needed, cause the DOM element on wich the scroll event listeners are attached are changed for every new model


            var modelId = 1;
            editorManager.setModelId(modelId);
            var dummyBPMModel = {
                "data": {
                    "modelId": "1",
                    "name": "My Process",
                    "key": "MyProcess",
                    "description": "My Process",
                    "lastUpdatedBy": "workflow",
                    "model": $rootScope.importedModel || {
                        "id": "canvas",
                        "resourceId": "canvas",
                        "stencilset": {
                            "namespace": "http://b3mn.org/stencilset/bpmn2.0#"
                        },
                        "properties": {
                            "process_id": "MyProcess",
                            "name": "My Process",
                            "documentation": "My Process"
                        },
                        "childShapes": [{
                            "bounds": {
                                "lowerRight": {
                                    "x": 130,
                                    "y": 193
                                },
                                "upperLeft": {
                                    "x": 100,
                                    "y": 163
                                }
                            },
                            "childShapes": [],
                            "dockers": [],
                            "outgoing": [],
                            "resourceId": "startEvent1",
                            "stencil": {
                                "id": "StartNoneEvent"
                            }
                        }],
                        "modelType": "model"
                    }
                },
                "status": 200,
                "config": {
                    "method": "GET",
                    "transformRequest": [null],
                    "transformResponse": [null],
                    "url": "/workflow-modeler/app/rest/models/7ace51f0-17e6-11ea-b4e5-00155d89ab02/editor/json?version=1575608952407",
                    "headers": {
                        "Accept": "application/json, text/plain, */*",
                        "Cache-Control": "no-cache, no-store, must-revalidate",
                        "Pragma": "no-cache",
                        "Expires": "0"
                    }
                },
                "statusText": ""
            }
            //we first initialize the stencilset used by the editor. The editorId is always the modelId.

            try {
                editorManager.setModelData(dummyBPMModel);
            } catch (ex) {
                alert("The input provided does not confirm to BPMN 2.0 specification");
            }
            var baseUrl = "http://b3mn.org/stencilset/";
            // Need to have copy of stencilJSON
            var tempStencilJson = jQuery.extend(true, {}, $rootScope.customPalette || stencilJson)
            editorManager.setStencilData(tempStencilJson);
            var stencilSet = new ORYX.Core.StencilSet.StencilSet(baseUrl, tempStencilJson);
            ORYX.Core.StencilSet.loadStencilSet(baseUrl, stencilSet, modelId);
            //after the stencilset is loaded we make sure the plugins.xml is loaded.
            ORYX._loadPlugins(require('raw-loader?esModule=false!./plugins.xml').toString());
            setTimeout(function () {
                editorManager.bootEditor();
            });

            //minihack to make sure mousebind events are processed if the modeler is used in an iframe.
            //selecting an element and pressing "del" could sometimes not trigger an event.
            jQuery(window).focus();

        }
    ]);

angular.module('workflowModeler')
    .controller('EditorUnsavedChangesPopupCtrl', ['$rootScope', '$scope', '$http', '$location', '$window', function ($rootScope, $scope, $http, $location, $window) {

        $scope.discard = function () {
            if ($scope.handleResponseFunction) {
                $scope.handleResponseFunction(true);
                // Also clear any 'onbeforeunload', added by oryx
                $window.onbeforeunload = undefined;
            }
            $scope.$hide();
        };

        $scope.save = function () {
            if ($scope.handleResponseFunction) {
                $scope.handleResponseFunction(false);
            }
            $scope.$hide();
        };

        $scope.cancel = function () {
            if ($scope.handleResponseFunction) {
                $scope.handleResponseFunction(null);
            }
            $scope.$hide();
        };
    }]);