/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
'use strict';
var Mousetrap = require('mousetrap');
angular.module('workflowModeler')
    .controller('ToolbarController', ['$scope', '$http', '$modal', '$q', '$rootScope', '$translate', '$location', 'editorManager',
        function ($scope, $http, $modal, $q, $rootScope, $translate, $location, editorManager) {
            $scope.editorFactory.promise.then(function () {
                var toolbarItems = WORKFLOW.TOOLBAR_CONFIG.items;
                $scope.items = [];
                var editSeparator = false;
                var rollbakSeparator = false;
                var alignmentSeparator = false;
                var zoomSeparator = false;
                var bendpointSeparator = false;
                var helpSeparator = false;
                for (var i = 0; i < toolbarItems.length; i++) {
                    if ($rootScope.modelData.model.modelType === 'form') {
                        if (!toolbarItems[i].disableInForm) {
                            $scope.items.push(toolbarItems[i]);
                        }
                    }
                    else {
                        const standAloneActions = ['WORKFLOW.TOOLBAR.ACTIONS.exportModel'
                            , 'WORKFLOW.TOOLBAR.ACTIONS.importProcess'
                            , 'WORKFLOW.TOOLBAR.ACTIONS.validate']
                        const currentActionIndex = standAloneActions.indexOf(toolbarItems[i].action);
                        if (currentActionIndex !== -1) {
                            if ($rootScope.showValidationBtn && standAloneActions[currentActionIndex] === 'WORKFLOW.TOOLBAR.ACTIONS.validate') {
                                $scope.items.push(toolbarItems[i]);
                            }
                            if ($rootScope.showXMLOptions && 
                                standAloneActions[currentActionIndex] === 'WORKFLOW.TOOLBAR.ACTIONS.exportModel') {
                                    $scope.items.push(toolbarItems[i]); 
                                }

                            if ($rootScope.showImport && 
                                standAloneActions[currentActionIndex] === 'WORKFLOW.TOOLBAR.ACTIONS.importProcess' ) {
                                $scope.items.push(toolbarItems[i]);
                            }
                            continue;
                        }
                        if ($rootScope.showCut &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.cut' ) {
                                $scope.items.push(toolbarItems[i]);
                                editSeparator = true;
                        }
                        else if ($rootScope.showCopy &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.copy' ) {
                                $scope.items.push(toolbarItems[i]);
                                editSeparator = true;
                        }
                        else if ($rootScope.showPaste &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.paste' ) {
                                $scope.items.push(toolbarItems[i]);
                                editSeparator = true;
                        }
                        else if ($rootScope.showDelete &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.deleteItem' ) {
                                $scope.items.push(toolbarItems[i]);
                                editSeparator = true;
                        }
                        else if ($rootScope.showRedo &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.redo' ) {
                                $scope.items.push(toolbarItems[i]);
                                rollbakSeparator = true;
                        }
                        else if ($rootScope.showUndo &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.undo' ) {
                                $scope.items.push(toolbarItems[i]);
                                rollbakSeparator = true;
                        }
                        else if ($rootScope.showAlignVertical &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.alignVertical' ) {
                                $scope.items.push(toolbarItems[i]);
                                alignmentSeparator = true;
                        }
                        else if ($rootScope.showAlignHorizontal &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.alignHorizontal' ) {
                                $scope.items.push(toolbarItems[i]);
                                alignmentSeparator = true;
                        }
                        else if ($rootScope.showSameSize &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.sameSize' ) {
                                $scope.items.push(toolbarItems[i]);
                                alignmentSeparator = true;
                        }
                        else if ($rootScope.showZoomIn &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.zoomIn' ) {
                                $scope.items.push(toolbarItems[i]);
                                zoomSeparator = true;
                        }
                        else if ($rootScope.showZoomOut &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.zoomOut' ) {
                                $scope.items.push(toolbarItems[i]);
                                zoomSeparator = true;
                        }
                        else if ($rootScope.showZoomActualSize &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.zoomActual' ) {
                                $scope.items.push(toolbarItems[i]);
                                zoomSeparator = true;
                        }
                        else if ($rootScope.showZoomFit &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.zoomFit' ) {
                                $scope.items.push(toolbarItems[i]);
                                zoomSeparator = true;
                        }
                        else if ($rootScope.showAddBendPoint &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.addBendPoint' ) {
                                $scope.items.push(toolbarItems[i]);
                                bendpointSeparator = true;
                        }
                        else if ($rootScope.showRemoveBendPoint &&
                            toolbarItems[i].action === 'WORKFLOW.TOOLBAR.ACTIONS.removeBendPoint' ) {
                                $scope.items.push(toolbarItems[i]);
                                bendpointSeparator = true;
                        }
                        else if ($rootScope.showHelp &&
                            toolbarItems[i].id === 'help-button' ) {
                                $scope.items.push(toolbarItems[i]);
                                helpSeparator = true;
                        }
                        else if (toolbarItems[i].type == "separator"){
                            $scope.items.push(toolbarItems[i]);
                        }
                    }
                }

                for(var i = 0; i < $scope.items.length; i++) {
                    if($scope.items[i].separatorPosition == "editSeparator" && !editSeparator) {
                        $scope.items.splice(i, 1);
                        i--;
                    } else if($scope.items[i].separatorPosition == "rollbakSeparator" && !rollbakSeparator){
                        $scope.items.splice(i, 1);
                        i--;
                    } else if($scope.items[i].separatorPosition == "alignmentSeparator" && !alignmentSeparator) {
                        $scope.items.splice(i, 1);
                        i--;
                    } else if($scope.items[i].separatorPosition == "zoomSeparator" && !zoomSeparator) {
                        $scope.items.splice(i, 1);
                        i--;
                    } else if($scope.items[i].separatorPosition == "bendpointSeparator" && !bendpointSeparator) {
                        $scope.items.splice(i, 1);
                        i--;
                    } else if($scope.items[i].separatorPosition == "helpSeparator" && !helpSeparator) {
                        $scope.items.splice(i, 1);
                        i--;
                    }
                }
			  
            });

			$scope.$on('undo', function() {
				$scope.triggerComponentEvents({action: 'undo'});
			});
			$scope.$on('redo', function() {
				$scope.triggerComponentEvents({action: 'redo'});
			});

			$scope.triggerComponentEvents = function(event) {
				var mapping = {
					'undo': 'WORKFLOW.TOOLBAR.ACTIONS.undo',
					'redo': 'WORKFLOW.TOOLBAR.ACTIONS.redo'
				}
				var currentAction = mapping[event.action];
				if (! currentAction) {
					return;
				}
				var items = WORKFLOW.TOOLBAR_CONFIG.items;

				for (var i = 0; i < items.length; i++) {
					if (items[i].action === currentAction) {
						var services = { '$scope': $scope, '$rootScope': $rootScope, '$http': $http, 
							'$modal': $modal, '$q': $q, '$translate': $translate, 'editorManager': editorManager };
						executeFunctionByName(items[i].action, WORKFLOW, services);
						// Other events
						var event = {
							type: WORKFLOW.eventBus.EVENT_TYPE_TOOLBAR_BUTTON_CLICKED,
							toolbarItem: items[i]
						};
						WORKFLOW.eventBus.dispatch(event.type, event);
						return ;
					}
				}
			};
			

            $scope.secondaryItems = WORKFLOW.TOOLBAR_CONFIG.secondaryItems;

            // Call configurable click handler (From http://stackoverflow.com/questions/359788/how-to-execute-a-javascript-function-when-i-have-its-name-as-a-string)
            var executeFunctionByName = function (functionName, context /*, args */) {
                var args = Array.prototype.slice.call(arguments).splice(2);
                var namespaces = functionName.split(".");
                var func = namespaces.pop();
                for (var i = 1; i < namespaces.length; i++) {
                    context = context[namespaces[i]];
                }
                return context[func].apply(this, args);
            };

            // Click handler for toolbar buttons
            $scope.toolbarButtonClicked = function (buttonIndex) {

                // Default behaviour
                var buttonClicked = $scope.items[buttonIndex];
                var services = { '$scope': $scope, '$rootScope': $rootScope, '$http': $http, '$modal': $modal, '$q': $q, '$translate': $translate, 'editorManager': editorManager };
                if (buttonClicked.action)
                    executeFunctionByName(buttonClicked.action, WORKFLOW, services);

                // Other events
                var event = {
                    type: WORKFLOW.eventBus.EVENT_TYPE_TOOLBAR_BUTTON_CLICKED,
                    toolbarItem: buttonClicked
                };
                WORKFLOW.eventBus.dispatch(event.type, event);
            };

            // Click handler for secondary toolbar buttons
            $scope.toolbarSecondaryButtonClicked = function (buttonIndex) {
                var buttonClicked = $scope.secondaryItems[buttonIndex];
                var services = { '$scope': $scope, '$http': $http, '$modal': $modal, '$q': $q, '$translate': $translate, '$location': $location, 'editorManager': editorManager };
                executeFunctionByName(buttonClicked.action, WORKFLOW, services);
            };

            /* Key bindings */
            Mousetrap.bind('mod+z', function (e) {
                if(!$rootScope.disableUndoRedo) {
                    if (document.body.classList.contains("modal-open")) {
                        return false;
                    }
                    var services = { '$scope': $scope, '$rootScope': $rootScope, '$http': $http, '$modal': $modal, '$q': $q, '$translate': $translate, 'editorManager': editorManager };
                    WORKFLOW.TOOLBAR.ACTIONS.undo(services);
                    return false;
                }
            });

            Mousetrap.bind('mod+y', function (e) {
                if(!$rootScope.disableUndoRedo) {
                    if (document.body.classList.contains("modal-open")) {
                       return false;
                    }
                    var services = { '$scope': $scope, '$rootScope': $rootScope, '$http': $http, '$modal': $modal, '$q': $q, '$translate': $translate, 'editorManager': editorManager };
                    WORKFLOW.TOOLBAR.ACTIONS.redo(services);
                    return false;
                }
            });

            Mousetrap.bind('mod+c', function (e) {
                if (document.body.classList.contains("modal-open")) {
                    return false;
                }
                var services = { '$scope': $scope, '$rootScope': $rootScope, '$http': $http, '$modal': $modal, '$q': $q, '$translate': $translate, 'editorManager': editorManager };
                WORKFLOW.TOOLBAR.ACTIONS.copy(services);
                return false;
            });

            Mousetrap.bind('mod+v', function (e) {
                if (document.body.classList.contains("modal-open")) {
                    return false;
                }
                var services = { '$scope': $scope, '$rootScope': $rootScope, '$http': $http, '$modal': $modal, '$q': $q, '$translate': $translate, 'editorManager': editorManager };
                WORKFLOW.TOOLBAR.ACTIONS.paste(services);
                return false;
            });

            Mousetrap.bind(['del'], function (e) {
                if (document.body.classList.contains("modal-open")) {
                    return false;
                }
                var services = { '$scope': $scope, '$rootScope': $rootScope, '$http': $http, '$modal': $modal, '$q': $q, '$translate': $translate, 'editorManager': editorManager };
                WORKFLOW.TOOLBAR.ACTIONS.deleteItem(services);
                return false;
            });

            Mousetrap.bind('mod+s', function (e) {
                return true; // Allowing event bubble up and parent application can catch the event in it.
            });

            /* Undo logic */

            $scope.undoStack = [];
            $scope.redoStack = [];

            WORKFLOW.eventBus.addListener(WORKFLOW.eventBus.EVENT_TYPE_UNDO_REDO_RESET, function ($scope) {
                this.undoStack = [];
                this.redoStack = [];
                if (this.items) {
                    for (var i = 0; i < this.items.length; i++) {
                        var item = this.items[i];
                        if (item.action === 'WORKFLOW.TOOLBAR.ACTIONS.undo' || item.action === "WORKFLOW.TOOLBAR.ACTIONS.redo") {
                            this.safeApply(function () {
                                item.enabled = false;
                            });
                        }
                    }
                }
            }, $scope);

            $scope.editorFactory.promise.then(function () {

                // Catch all command that are executed and store them on the respective stacks
                editorManager.registerOnEvent(ORYX.CONFIG.EVENT_EXECUTE_COMMANDS, function (evt) {

                    // If the event has commands
                    if (!evt.commands) { return; }

                    $scope.undoStack.push(evt.commands);
                    $scope.redoStack = [];

                    for (var i = 0; i < $scope.items.length; i++) {
                        var item = $scope.items[i];
                        if (item.action === 'WORKFLOW.TOOLBAR.ACTIONS.undo') {
                            item.enabled = true;
                        }
                        else if (item.action === 'WORKFLOW.TOOLBAR.ACTIONS.redo') {
                            item.enabled = false;
                        }
                    }

                    // Update
                    editorManager.getCanvas().update();
                    editorManager.updateSelection();

                });

            });

            // Handle enable/disable toolbar buttons 
            $scope.editorFactory.promise.then(function () {
                editorManager.registerOnEvent(ORYX.CONFIG.EVENT_SELECTION_CHANGED, function (evt) {
                    var elements = evt.elements;

                    for (var i = 0; i < $scope.items.length; i++) {
                        var item = $scope.items[i];
                        if (item.enabledAction && item.enabledAction === 'element') {
                            var minLength = 1;
                            if (item.minSelectionCount) {
                                minLength = item.minSelectionCount;
                            }

                            if (elements.length >= minLength && !item.enabled) {
                                $scope.safeApply(function () {
                                    item.enabled = true;
                                });
                            } else if (elements.length == 0 && item.enabled) {
                                $scope.safeApply(function () {
                                    item.enabled = false;
                                });
                            }
                        }
                    }
                });

            });

        }]);
