import bootStrapStyles from './libs/bootstrap_3.4.1/css/bootstrap.min.css';
import gridStyles from 'angular-ui-grid/ui-grid.css';
import cssStyles from './styles/workflow-modeler.css';
import fontsCSS from './styles/fonts.css';
import fontsIconsCSS from './icons.font';

import editorHTML from './editor-app/partials/editor.html';
const template = `
        <div id='modeler' class='ot-modeler' ng-cloak="">
               <div id="main-header" class="header" ng-if='standalone'>
                    <i class='logo ot-icons ot-icon-opentext_logo'></i>
                </div>
                <div id="main" ng-if='!hideEditor' class="wrapper full clearfix" ng-controller='EditorController' ng-style="{height: window.height + 'px'}">
                    ${editorHTML}
                </div>
         </div>
               `
export class WorkflowModeler extends HTMLElement {
  
    constructor() {
        // always call super() first 
        super();

        this.defaultConfig = JSON.parse(`{
            "showValidation": false,
            "showExport": false,
            "showSidePanels": true,
            "helpDocumentLink": "",
            "showImport": false,
            "showCut": true,
            "showCopy": true,
            "showPaste": true,
            "showDelete": true,
            "showRedo": true,
            "showUndo": true,
            "disableUndoRedo": false,
            "showAlignVertical": true,
            "showAlignHorizontal": true,
            "showSameSize": true,
            "showZoomIn": true,
            "showZoomOut": true,
            "showZoomActualSize": true,
            "showZoomFit": true,
            "showAddBendPoint": true,
            "showRemoveBendPoint": true,
            "showHelp": true,
            "showTargetNameSpaceAsDropdown": false
		}`);

        this.initialized = false;
        window.rootEle = this._shadowRoot = this.attachShadow({ 'mode': 'open' });
        this._shadowRoot.innerHTML = template;
        this.prevId = "";

        //overriding the prototype function
        window.$$ = function () {
            const expression = $A(arguments).join(', ');
            return Prototype.Selector.select(expression, window.rootEle.querySelector('#modeler'));
        };
        this.triggerLoadEvent = this.triggerLoadEvent.bind(this);
        this.triggerChangeEvent = this.triggerChangeEvent.bind(this);
        this.undo = this.undo.bind(this);
        this.redo = this.redo.bind(this);
        this.registerEvents();
        this.defaultModel = {
            "id": "canvas",
            "resourceId": "canvas",
            "stencilset": {
                "namespace": "http://b3mn.org/stencilset/bpmn2.0#"
            },
            "properties": {
                "process_id": "MyProcess",
                "name": "My Process",
                "documentation": "My Process"
            },
            "childShapes": [{
                "bounds": {
                    "lowerRight": {
                        "x": 130,
                        "y": 193
                    },
                    "upperLeft": {
                        "x": 100,
                        "y": 163
                    }
                },
                "childShapes": [],
                "dockers": [],
                "outgoing": [],
                "resourceId": "startEvent1",
                "stencil": {
                    "id": "StartNoneEvent"
                }
            }],
            "modelType": "model"
        }
    }

    initialize(json) {
        if (!this.lazyInit) {
            return null;
        }

        if (this.initialized) {
            return null;
        }

        this.config = { ...this.defaultConfig, ...json }
        this.bootStrapAngularApp();
    }

	registerEvents() {
        this.registerForLoadEvent();
        this.registerForChangeEvents();
        this.registerForUndoEvents();
        this.registerForRedoEvents()
	}

    populateEndpoints(endPoints) {
        const $rootScope = this.getAngularRootScope();
        if (endPoints.validateProcessModel) {
            $rootScope.validateURL = endPoints.validateProcessModel
        }

        if (endPoints.convertJsonToBPMNXml) {
            $rootScope.convertToXmlUrl = endPoints.convertJsonToBPMNXml;
        }

        if (endPoints.importProcessModelXml) {
            $rootScope.importXmlUrl = endPoints.importProcessModelXml;
        }
    }

    attachStyles() {
        const iconsFonts = fontsIconsCSS.toString();
        const index = iconsFonts.indexOf('}');
        const iconsFontsFamily = iconsFonts.substring(0, index + 1);
        const iconsFontCss = iconsFonts.substring(index + 1, iconsFonts.length);

        const styleSheet = document.createElement('style');
        styleSheet.append(bootStrapStyles.toString());
        styleSheet.append(gridStyles.toString());
        styleSheet.append(cssStyles.toString());
        styleSheet.append(iconsFontCss);
        this._shadowRoot.appendChild(styleSheet);
        // adding the font the parent element 
        const fontStyle = document.createElement('style');
        fontStyle.append(fontsCSS.toString());
        fontStyle.append(iconsFontsFamily);
        this.appendChild(fontStyle);
    }

    connectedCallback() {
        if (!this.hasAttribute('standalone')) {
            this.setAttribute('standalone', false);
        } else {
            this.standalone = (this.getAttribute('standalone') == '' || this.getAttribute('standalone') == "true") ? true : false;
        }

        if (!this.hasAttribute('load-user-config')) {
            this.lazyInit = false;
        } else {
            this.lazyInit = true;
            const lazyInit = this.getAttribute('load-user-config');
            if (lazyInit === false || lazyInit == 'false') {
                this.lazyInit = false;
            }
        }
        this.attachStyles();
        if (!this.lazyInit) {
            this.config = {...this.defaultConfig};
            if (this.standalone) {
                this.config.showExport = true;
                this.config.showImport = true;
                this.config.showValidation = true;
                this.config.showTargetNameSpaceAsDropdown = false;
            }
            this.bootStrapAngularApp();
        }
    }

    bootStrapAngularApp() {
        // bootstrap at
        const workflowModule = angular.module('workflowModeler');
        workflowModule.run(['$rootScope', $rootScope => {
            $rootScope.showValidationBtn = this.config.showValidation;
            $rootScope.showXMLOptions = this.config.showExport;
            $rootScope.showSidePanels = this.config.showSidePanels;
            $rootScope.showImport = this.config.showImport;
            $rootScope.showCut = this.config.showCut;
            $rootScope.showCopy = this.config.showCopy;
            $rootScope.showPaste = this.config.showPaste;
            $rootScope.showDelete = this.config.showDelete;
            $rootScope.showRedo = this.config.showRedo;
            $rootScope.showUndo = this.config.showUndo;
            $rootScope.disableUndoRedo = this.config.disableUndoRedo;
            $rootScope.showAlignVertical = this.config.showAlignVertical;
            $rootScope.showAlignHorizontal = this.config.showAlignHorizontal;
            $rootScope.showSameSize = this.config.showSameSize;
            $rootScope.showZoomIn = this.config.showZoomIn;
            $rootScope.showZoomOut = this.config.showZoomOut;
            $rootScope.showZoomActualSize = this.config.showZoomActualSize;
            $rootScope.showZoomFit = this.config.showZoomFit;
            $rootScope.showAddBendPoint = this.config.showAddBendPoint;
            $rootScope.showRemoveBendPoint = this.config.showRemoveBendPoint;
            $rootScope.showSidePanels = this.config.showSidePanels;
            $rootScope.showHelp = this.config.showHelp;
            $rootScope.helpDocumentLink = this.config.helpDocumentLink;
            $rootScope.showTargetNameSpaceAsDropdown = this.config.showTargetNameSpaceAsDropdown;
            if (this.config.endPointURLs) {
                const endPoints = this.config.endPointURLs

                if (endPoints.validateProcessModel) {
                    $rootScope.validateURL = endPoints.validateProcessModel
                }

                if (endPoints.convertJsonToBPMNXml) {
                    $rootScope.convertToXmlUrl = endPoints.convertJsonToBPMNXml;
                }

                if (endPoints.importProcessModelXml) {
                    $rootScope.importXmlUrl = endPoints.importProcessModelXml;
                }
                if (endPoints && endPoints.headers) {
                    $rootScope.headers = endPoints.headers;
                }
            }

            if (this.config.customPaletteJSON) {
                $rootScope.customPalette = this.config.customPaletteJSON;
            }

            if (this.config.customCSSURL) {
                this.appendCustomCss(this.config.customCSSURL);
            }

            if (this._processModelJson) {
                $rootScope.importedModel = this._processModelJson;
                this._processModelJson = null;
            }
            setTimeout(function () {
                $rootScope.$apply(function () {
                    $rootScope.hideEditor = false;
                });
            })
        }]);
        angular.bootstrap(this._shadowRoot.querySelector('#modeler'), ['workflowModeler']);
        this.initialized = true;
        this.applyToAngular();
    }

    registerForLoadEvent() {
        this._shadowRoot.addEventListener('modelerLoaded', this.triggerLoadEvent);
    }

    triggerLoadEvent() {
        const event = new Event('load', {bubbles: true});
        this.dispatchEvent(event);
    }

    registerForChangeEvents() {
        this._shadowRoot.addEventListener('modelChange', this.triggerChangeEvent);
    }

    triggerChangeEvent(eventData) {
        const event = new Event('change', { bubbles: true});
        event.data = {isInitialState:eventData.detail.isInitialState};
        this.dispatchEvent(event);
    }

	registerForUndoEvents() {
		this.addEventListener('undo',  this.undo);
	}

	undo () {
		const $rootScope = this.getAngularRootScope();
		$rootScope && $rootScope.$broadcast('undo');
	}

	registerForRedoEvents() {
		this.addEventListener('redo', this.redo);
	}

	redo() {
		const $rootScope = this.getAngularRootScope();
		$rootScope && $rootScope.$broadcast('redo');
	}

    resetUndoRedo () { // User will call when downlod the modeler json.
        WORKFLOW.eventBus.dispatch(WORKFLOW.eventBus.EVENT_TYPE_UNDO_REDO_RESET, {});
    }

    static get observedAttributes() {
        return ['standalone'];
    }

    attributeChangedCallback(name, oldValue, newValue) {
        switch (name) {
            case 'standalone':
                this.applyToAngular();
        }
    }

    get standalone() {
        const value = this.getAttribute('standalone');
        return value !== 'false';
    }

    set standalone(value) {
        this.setAttribute('standalone', value);
    }

    get processModelJSON() {
        const $root = this._shadowRoot.querySelector('#modeler');
        const $editorManager = angular.element($root).injector().get('editorManager');
        return $editorManager.getModel();
    }

    set processModelJSON(json) {
        if (!this.initialized) {
            this._processModelJson = json;
            return;
        }
        const $root = this._shadowRoot.querySelector('#modeler');
        const $rootScope = angular.element($root).injector().get('$rootScope');
        $rootScope.$apply(() => {
            $rootScope.editorInitialized = false;
            $rootScope.importedModel = json;
        });
    }

    get paletteJSON() {
        const $rootScope = this.getAngularRootScope();
        if ($rootScope == null) return null;
        return $rootScope.customPalette;
    }

    //user context
    set context(context) {
        this.context = context;
    }

    get context() {
        return this.context;
    }

    getAngularRootScope() {
        if (!this.initialized) {
            return null;
        }
        const $root = this._shadowRoot.querySelector('#modeler');
        const $rootScope = angular.element($root).injector().get('$rootScope');
        return $rootScope;
    }

    appendCustomCss(cssUrl) {
        const link = document.createElement("link");
        link.type = "text/css";
        link.rel = "stylesheet";
        link.href = cssUrl;

        this._shadowRoot.appendChild(link);
    }

    applyToAngular() {
        if (!this.initialized) {
            return;
        }
        const $rootScope = this.getAngularRootScope();
        $rootScope.$apply(() => { $rootScope.standalone = this.standalone; });
    }

    disconnectedCallback() {
        delete window.rootEle;
        this._shadowRoot.removeEventListener('modelerLoaded', this.triggerLoadEvent);
        this._shadowRoot.removeEventListener('modelChange', this.triggerChangeEvent);
    }

    selectStencil(objId) {
        try {
            if(this.prevId && rootEle.querySelectorAll("#" + this.prevId)[0]){
                rootEle.querySelectorAll("#" + this.prevId)[0].style.outline = "none";
            }
            this.prevId = objId;
            if(this.prevId && rootEle.querySelectorAll("#" + this.prevId)[0]){
                rootEle.querySelectorAll("#" + this.prevId)[0].style.outline = "2px solid red";
                rootEle.querySelectorAll("#" + this.prevId)[0].scrollIntoViewIfNeeded();
            }
        } catch(e){
            console.log(e.message);
        }
    }
  
    showErrors(json) {
      const $rootScope = this.getAngularRootScope();
      if ($rootScope && typeof json == 'object') {
        $rootScope.showErrorInfo(json);
      }
    }

    registerNameSpaceProvider(getNameSpaces = () => {}) {
       return window.getNameSpacesFn = getNameSpaces;
    }

    /* Unload the model and clears the WORKFLOW consumption memory */
    unloadProcessModel() {
      window.WORKFLOW.eventBus.listeners = {};
    }
}

window.customElements.define('workflow-modeler', WorkflowModeler);
