/* UTILITY METHODS */

/**
 * This creates a modal window that auto closes on route change.
 * By default, this is NOT the case, and leads to some funny behaviour.
 *
 * Use this method vs the default $modal({myJson}) approach
 */
_internalCreateModal = function(modalConfig, $modal, $scope) {
    modalConfig.container = angular.element(window.rootEle);
    modalConfig.backdrop = 'static';
    setTimeout(function () {
        $scope.modal.$element.find('.ot-icon-close')[0].focus();
        $scope.modal.$element.find('.btn-secondary').on("keydown", function (event) {
            if (event.keyCode == 9 && !event.shiftKey) {
                event.preventDefault();
                $scope.modal.$element.find('.ot-icon-close')[0].focus();
            }
        });
        $scope.modal.$element.find('.btn-primary').on("keydown", function (event) {
            if (event.keyCode == 9 && !event.shiftKey && $scope.modal.$element.find('.btn-primary').is(':last-child')) {
                event.preventDefault();
                $scope.modal.$element.find('.ot-icon-close')[0].focus();
            }
        });
        $scope.modal.$element.find('.ot-icon-close').on("keydown", function (event) {
            if (event.keyCode == 9 && event.shiftKey) {
                event.preventDefault();
                if ($scope.modal.$element.find('.btn-secondary')[0] != undefined) {
                    $scope.modal.$element.find('.btn-secondary')[0].focus();
                } else {
                    $scope.modal.$element.find('.btn-primary')[0].focus();
                }

            }
        });
    }, 500)
    if (typeof modalConfig.template === 'string' 
    && modalConfig.template.indexOf('?version=') !== -1) {
        modalConfig.template = modalConfig.template.substring(0, modalConfig.template.indexOf('?version='))
    }
    if ($scope !== null && $scope !== undefined) {
        $scope.modal = $modal(modalConfig);

        $scope.$on('$routeChangeStart', function () {
            if ($scope.modal) {
                $scope.modal.hide();
            }
        });

        return $scope.modal;
    } else {
        return $modal(modalConfig);
    }
};
var workflowModeler = angular.module('workflowModeler');
workflowModeler.directive('tabControl', ['$compile', '$http', '$templateCache', function($compile, $http, $templateCache) {

    var updateTemplate = function($scope, element, attributes) {
        if(!$scope.activeTemplate || $scope.activeTemplate != $scope.activeTab.id) {
            // Check if current loaded template is still the right one
            var contentDiv = $(element.children()[1]);

            var childScope = angular.element(element.children()[1]).scope();
            if($scope.activeTemplate && childScope != $scope) {
                // Child-scope created by the included element, should be destroyed
                childScope.$destroy();
            }

            if($scope.activeTab && $scope.activeTab.templateUrl) {
                // Load the HTML-fragment or get from cache
                var loader = $http.get($scope.activeTab.templateUrl, {cache: $templateCache});
                var promise = loader.success(function(html) {
                    contentDiv.html(html);
                }).then(function (response) {
                    $scope.activeTemplate = $scope.activeTab.id;
                    contentDiv.replaceWith($compile(contentDiv.html())($scope));
                });
            } else {
                // No templates are being used, no need to use the contentDiv for this tab, clear it
                contentDiv.empty();
            }
        }
    };

    var directive = {};
    directive.restrict = 'A';
    directive.replace = true;
    directive.transclude = true;
    directive.template = '<div><div class="clearfix"><ul class="tabs clearfix">' +
        '<li ng-repeat="tab in tabs" ng-class="{\'active\': tab.id == activeTab.id, \'disabled\': tab.disable}"><a ng-click="tabClicked(tab)" ng-keypress="tabKeyPress($event,tab)" tabindex="{{tab.tabindex}}">{{tab.title && (tab.title | translate) || (tab.name | translate)}} <div class="title"></div></a></li>' +
        '</ul></div>' +
        '<div></div>' +
        '</div>';

    directive.scope = {
        possibleTabs : "=tabControl",
        model: "=model",
        activeTabReference: "=activeTab"
    };


    directive.controller = ['$scope', '$element', function($scope, $element) {

        $scope.refreshTabs = function() {
            var tabs = [];
            for(var i=0; i < $scope.possibleTabs.length; i++) {
                var tab = $scope.possibleTabs[i];
                tab.tabindex = !tab.disable ? 0 : -1;
                if(!tab.hide) {
                    tabs.push(tab);
                }
            }
            $scope.tabs = tabs;
        };

        $scope.$watch('possibleTabs', function () {
            $scope.refreshTabs();
        }, true);

        $scope.$watch('activeTabReference', function(newValue, oldValue) {
            if(!$scope.activeTab || $scope.activeTab.id != newValue) {
                // Active tab ID changed from outside of the directive controller, need to switch to the
                // right tab within this scope
                var newTab = $scope.findTab(newValue);
                if(newTab) {
                    $scope.tabClicked(newTab);
                }
            }
        });

        $scope.findTab = function(tabId) {
            if($scope.possibleTabs) {
                for(var i=0; i< $scope.possibleTabs.length; i++) {
                    if($scope.possibleTabs[i].id == tabId) {
                        return $scope.possibleTabs[i];
                    }
                }
            }
            return undefined;
        };

        $scope.tabClicked = function(tab) {

            if (tab.disable) {
                return ;
            }

            if (tab.hide) {
                tab.hide = false;
                $scope.refreshTabs();
            }
            $scope.activeTab = tab;
            if (tab) {
                $scope.activeTabReference = tab.id;
            } else {
                $scope.activeTabReference = undefined;
            }
            updateTemplate($scope, $element);
        };

        $scope.tabKeyPress = function (event, tab) {
            if (event.keyCode == 13 || event.keyCode == 32) {
                $scope.tabClicked(tab);
            }
        }
        $scope.refreshTabs();

        if($scope.tabs && $scope.tabs.length > 0) {
            if($scope.activeTabReference) {
                $scope.activeTab = $scope.findTab($scope.activeTabReference);
            }

            if(!$scope.activeTab) {
                // Revert to the first tab, if no tab is forced to be shown first
                $scope.activeTab = $scope.tabs[0];
            }
            $scope.tabClicked($scope.activeTab);
        }
    }];

    directive.link = updateTemplate;

    return directive;
}]);



workflowModeler.service('toastService', ['$rootScope', function ($rootScope) {
        this.error = function (errMsg) {

            var toast  = {
                type: 'error',
                msg: errMsg,
                timeout: 5000
            }

            $rootScope.$emit('add-toaster-error', toast);

            var clearToast = function () {
                $rootScope.$emit('clear-toaster-error', toast);
            }

            return clearToast;
        }
}]);

workflowModeler.directive('toastDirective', ['$rootScope', '$templateCache', function($rootScope, $templateCache) {

    return {
        restrict: 'E',
        scope: true,
        template: $templateCache.get('editor-app/configuration/properties/toster.html'),
        controller: ['$scope', '$element', '$timeout', function ($scope, $element, $timeout) {
            $scope.toasts = [];

            $rootScope.$on('add-toaster-error', function(evt , toast) {
                $scope.toasts.push(toast);
                $timeout(function() {
                    removeToast(toast);
                }, toast.timeout);
            });

            function removeToast(toast) {
                var index = $scope.toasts.indexOf(toast);
    
                if (index !== -1) {
                    $scope.toasts.splice(index, 1);
                }
            }

            $rootScope.$on('clear-toaster-error', function (evt, toast) {
                $timeout(function () {
                    removeToast(toast);
                });
            });
        }]
    }



}]);

