const TIME_OUT = 150;
export class TestUtils {


	/**
	 * Renders workflow modeler webcomponents depend on the mode of render
	 * waits for the modeler to finish and returns promise
	 *
	 */
	static async renderModeler(props) {
		const timeout = props['load-user-config'] === true ? 0 : TIME_OUT;
		return new Promise(async (resolve, reject) => {
		 	const component = await TestUtils.render('workflow-modeler', props);
			setTimeout(() => {
				resolve(component);
			}, timeout);
		});
	}


	/**
	 * Callse componets(modeler) intialize method with props passed in
	 * and waits for the angular to complete its intialize
	 * and rutrns promise
	 *
	 */
	static async initializeModeler(component, props = {}) {
		return new Promise((resolve, reject) => {
			component.initialize(props);
			setTimeout(() => {
				resolve(component);
			}, TIME_OUT);
		})
	}



	/**
	 * Renders a given element with provided attributes
	 * and returns a promise which resolves as soon as
	 * rendered element becomes available.
	 * @param {string} tag
	 * @param {object} attributes
	 * @returns {Promise<HTMLElement>}
	 */
	static render(tag, attributes = {}) {
		TestUtils._renderToDocument(tag, attributes);
		return TestUtils._waitForComponentToRender(tag);
	}

	/**
	 * Replaces document's body with provided element
	 * including given attributes.
	 * @param {string} tag
	 * @param {object} attributes
	 */
	static _renderToDocument(tag, attributes) {
		const htmlAttributes = TestUtils._mapObjectToHTMLAttributes(attributes);
		document.body.innerHTML = `<${tag} ${htmlAttributes}></${tag}>`;
	}

	/**
	 * Converts an object to HTML string representation of attributes.
	 *
	 * For example: `{ foo: "bar", baz: "foo" }`
	 * becomes `foo="bar" baz="foo"`
	 *
	 * @param {object} attributes
	 * @returns {string}
	 */
	static _mapObjectToHTMLAttributes(attributes) {
		return Object.entries(attributes).reduce((previous, current) => {
			return previous + ` ${current[0]}="${current[1]}"`;
		}, "");
	}

	/**
	 * Returns a promise which resolves as soon as
	 * requested element becomes available.
	 * @param {string} tag
	 * @returns {Promise<HTMLElement>}
	 */
	static async _waitForComponentToRender(tag) {
		return new Promise(resolve => {
			function requestComponent() {
				const element = document.querySelector(tag);
				if (element)  {
					setTimeout(() => {
						resolve(element);
					});

				} else {
					window.requestAnimationFrame(requestComponent);
				}
			}
			requestComponent();
		});
	}

	static removeComponent(node) {
		node.parentElement.removeChild(node)
	}
}
