import { WorkflowModeler } from '../src/root.es6';
import { TestUtils } from '../tests/TestUtils';

describe ("WorkflowModeler web-component", () => {

	let workflow;
	beforeEach(() => {
		workflow = new WorkflowModeler();
	});

	it ("should be instance of the HTMLElment", () => {
		expect(workflow instanceof HTMLElement).toBe(true);
	});

	it ("should have create open shadow-dom and assing to rootEle on windowObject", () => {
		expect(window.rootEle).toEqual(workflow._shadowRoot);	
	});


	describe("intialize method", () => {

		let component;

		beforeEach(() => {
			component = new WorkflowModeler();
		});

		it ("should return null if lazyInit is undefined", () => {
			component.lazyInit = null;
			expect(component.initialize({})).toEqual(null);
		});


		it ("should return null if it is called more than once", () => {
			spyOn(component, 'bootStrapAngularApp')
				.and.callFake(() =>{
					component.initialized = true;
				});
			spyOn(component, 'getAngularRootScope')
				.and.returnValue({
					$apply: () => {
						return null;
					}
				});
			component.lazyInit = true;
			var _ = component.initialize({});
			expect(component.initialize({})).toEqual(null);
		})
	});


	describe("getAngularRootScope method", () => {
		
		let _root;
		beforeEach(() => {
			_root = new WorkflowModeler();
		});
		xit ("should return null on the lazy init (load-user-config) is enabled component before calling intialize method and should return angular $rootScope once component is intailized", async () => {
			_root = await TestUtils.renderModeler({'load-user-config': true});
			expect(_root.getAngularRootScope()).toBeNull();
			_root = await TestUtils.initializeModeler(_root,{});
			const angularRootScope = angular.element(_root._shadowRoot.querySelector('#modeler')).injector().get('$rootScope');
			expect(_root.getAngularRootScope()).toBe(angularRootScope);
		});



		xit ("should return angular $rootScope if the app is intialized" , async () => {
			_root = await TestUtils.renderModeler({'load-user-config': false});
			const angularRootScope = angular.element(_root._shadowRoot.querySelector('#modeler')).injector().get('$rootScope');
			expect(_root.getAngularRootScope()).toBe(angularRootScope);
		})

		afterEach(() => {
			TestUtils.removeComponent(_root);
		});
	});
});
