const path = require('path');
const webpack = require('webpack');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const { CleanWebpackPlugin } = require('clean-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const angularLibsPath = require('./angularLibPaths');

const htmlPluginConfig = {
    title: 'Workflow Modeler',
    template: 'index.html'
}

const webcomponentsjs = './node_modules/@webcomponents/webcomponentsjs';
const polyfills = [
    {
        from: path.resolve(`${webcomponentsjs}/webcomponents-*.{js,map}`),
        to: path.join(__dirname, 'dist'),
        flatten: true
    },
    {
        from: path.resolve(`${webcomponentsjs}/bundles/*.{js.map}`),
        to: path.resolve(__dirname, 'dist'),
        flatten: true
    },
    {
        from: path.resolve(`${webcomponentsjs}/custom-elements-es5-adapter.js`),
        to: path.join(__dirname, 'dist'),
        flatten: true
    },
    {
        from: path.resolve('./node_modules/@babel/polyfill/dist/polyfill.min.js'),
        to: path.join(__dirname, 'dist'),
        flatten: true
    }
]
const CopyPackageDeploy = [{
    from: path.resolve('./package-deploy.json'),
    to: path.join(__dirname, 'dist/package.json'),
    flatten: true
  },
  {
    from: path.resolve('./.npmrc'),
    to: path.join(__dirname, 'dist/'),
    flatten: true
  }
];

const loadHelperDocs = [{
    from: path.resolve('./src/docs/Workflow_modeler_constructs_properties.pdf'),
    to: path.join(__dirname, 'dist'),
    flatten: true
}];

const workflow_icons = [
    {
        from: path.resolve('./src/otfavicon.ico'),
        to: path.join(__dirname, 'dist'),
        flatten: true
    }
];

module.exports = {
    context: path.resolve(__dirname, 'src'),

    entry: {
        "workflow-modeler": './entry.js'
    },

    mode: 'development',

    plugins: [
        new HtmlWebpackPlugin(htmlPluginConfig),
        new webpack.ProvidePlugin({
            WORKFLOW: path.resolve(__dirname, './src/scripts/app-cfg'),
            'window.jQuery': 'jquery',
            jQuery: 'jquery',
            ORYX: path.resolve(__dirname, './src/proxyORYX')
        }),
        new CopyWebpackPlugin([...polyfills, ...loadHelperDocs, ...workflow_icons, ...CopyPackageDeploy])
    ],

    output: {
        filename: '[name].js',
        path: path.resolve(__dirname, 'dist')
    },


    // only for the development purpose , Impacts the web performance
    devtool: 'inline-source-map',

    devServer: {
        contentBase: path.join(__dirname, 'dist'),
        port: 9006,
        watchContentBase: true,
        proxy: {
            '/app/*': 'http://localhost:9000/workflow-modeler'
        }
    },

    resolve: {
        extensions: [' ', '.js', '.es6'],
        alias: {
            'jQuery': 'jquery',
            'WORKFLOW': path.resolve(__dirname, './src/scripts/app.cfg'),
            'ORYX': path.resolve(__dirname, './src/proxyORYX'),
            'PathParser': path.resolve(__dirname, './src/editor-app/libs/path_parser'),
            'angular': path.resolve(__dirname, './src/angularProxy'),
            'windowProxy': path.resolve(__dirname, './src/windowProxy')
        }
    },

    module: {

        rules: [
            {
                test: /\.?es6$/,
                exclude: /node_modules/,
                use: [
                    {
                        loader: 'imports-loader',
                        options: {
                            angular: 'angular'
                        }
                    },
                    {
                        loader: 'babel-loader',
                        options: {
                            presets: ['@babel/preset-env']
                        }
                    }]
            },{
                test: /\.css$/i,
                include: [
                    path.resolve(__dirname, 'node_modules/angular-ui-grid/ui-grid.css'),
                    path.resolve(__dirname, 'src/libs/bootstrap_3.4.1/css/bootstrap.min.css')
                ],
                use : [{ loader: 'css-loader', options: { importLoaders: 1, url: false }}]
            },{
                test: /\.css$/i,
                exclude: [
                    path.resolve(__dirname, 'node_modules/angular-ui-grid/ui-grid.css'),
                    path.resolve(__dirname, 'src/libs/bootstrap_3.4.1/css/bootstrap.min.css')
                ],
                use: [{ loader: 'css-loader', options: { importLoaders: 1 } },
                    'postcss-loader'
                ],
            }, {
                test: /\.font\.js/,
                use: [
                    'css-loader',
                    'webfonts-loader'
                ]
            },{
                test: /\.(gif|png|jpe?g|svg)$/i,
                use: [
                    {
                        loader: 'url-loader'
                    },
                    {
                        loader: 'image-webpack-loader',
                        options: {
                            mozjpeg: {
                                progressive: true,
                                quality: 65
                            },
                            // optipng.enabled: false will disable optipng
                            optipng: {
                                enabled: false,
                            },
                            pngquant: {
                                quality: [0.65, 0.90],
                                speed: 4
                            },
                            gifsicle: {
                                interlaced: false,
                            },
                            // the webp option will enable WEBP
                            webp: {
                                quality: 75
                            }
                        },
                    },

                ],
            }, {
                test: /\.js$/,
                include: angularLibsPath,
                use: [
                    {
                        loader: 'imports-loader',
                        options: {
                            ORYX: 'ORYX',
                            window: 'windowProxy',
                            angular: 'angular',
                            'window.jQuery': 'jquery',
                            'window.angular': 'angular'
                        }
                    }
                ]
            }, {
                test: /\.js$/,
                exclude: [
                    /\.font\.js/,
                    /node_modules/,
                    path.resolve(__dirname, 'node_modules/angular/angular'),
                    path.resolve(__dirname, './src/angularProxy'),
                    path.resolve(__dirname, './src/windowNodeProxy'),
                    path.resolve(__dirname, './src/windowProxy'),
                    ...angularLibsPath
                ],
                use: [
                    {
                        loader: 'required-loader',
                        options: {
                            import: 'angular',
                        }
                    }, {
                        loader: 'imports-loader',
                        options: {
                            ORYX: 'ORYX',
                            angular: 'angular',
                        }
                    }
                ]
            },
            {
                test: /\.html$/,
                include: [
                    path.join(__dirname, 'src/editor-app/configuration/properties'),
                ],
                use: [
                    {
                        loader: 'imports-loader',
                        options: {
                            'window.angular': 'angular'
                        }
                    }, {
                        loader: 'ng-cache-loader?prefix=[dir]/[dir]/[dir]&module=workflowModeler'
                    }
                ]
            },
            {
                test: /\.html$/,
                include: [
                    path.join(__dirname, 'src/editor-app/partials'),
                    path.join(__dirname, 'src/editor-app/popups')
                ],
                use: [
                    {
                        loader: 'imports-loader',
                        options: {
                            'window.angular': 'angular'
                        }
                    },
                    {
                        loader: 'ng-cache-loader?prefix=[dir]/[dir]&module=workflowModeler'
                    }
                ]
            }
        ]

    }
}
